SystemOrganization addCategory: #RoarVM!
SystemOrganization addCategory: #'RoarVM-Misc'!
SystemOrganization addCategory: #'RoarVM-Monitoring'!

!Process class methodsFor: '*RoarVM' stamp: 'ssa 2/26/2009 02:45'!
forContext: aContext priority: anInteger coreMask: anIntegerMask
	"Answer an instance of me that has suspended aContext at priority 
	anInteger using this coreMask."

	| newProcess |
	newProcess := self forContext: aContext priority: anInteger.
	newProcess coreMask:anIntegerMask.
	^newProcess! !

!Process methodsFor: '*RoarVM' stamp: 'ssa 2/26/2009 02:57'!
avoidCore: anIndex
	"Force this process to avoid this core (0-55)"
	self coreMask: (self coreMask bitClear: (1 bitShift: anIndex))! !

!Process methodsFor: '*RoarVM' stamp: 'ssa 2/26/2009 03:49'!
avoidCores: aCollectionOfIndicies
	"Force this process to avoid these cores (0-55)"
	| mask |
	mask := 0.
	aCollectionOfIndicies do:[:index| mask := mask bitOr: (1 bitShift: index)].
	self coreMask: (self coreMask bitClear:mask)! !

!Process methodsFor: '*RoarVM' stamp: 'dmu 2/12/2009 23:15'!
coreMask ^ coreMask! !

!Process methodsFor: '*RoarVM' stamp: 'dmu 2/12/2009 23:15'!
coreMask: m coreMask := m! !

!Process methodsFor: '*RoarVM' stamp: 'dmu 2/12/2009 23:14'!
hostCore ^ hostCore! !

!Process methodsFor: '*RoarVM' stamp: 'dmu 3/8/2010 20:26'!
hostCore: anObject
	"Set the value of hostCore"

	hostCore := anObject! !

!Process methodsFor: '*RoarVM' stamp: 'dmu 4/7/2009 02:41'!
resumeOn: aCore
  self useOnlyCore: aCore.
  self resume! !

!Process methodsFor: '*RoarVM' stamp: 'ssa 2/26/2009 02:59'!
useAdditionalCore: anIndex
	"Force this process to only urn on this core (0-55)"
	self coreMask: (self coreMask bitOr:(1 bitShift: anIndex))! !

!Process methodsFor: '*RoarVM' stamp: 'ssa 2/26/2009 03:50'!
useAdditionalCores: aCollectionOfIndicies
	"Allow this process to run on these additional cores (0-55)"
	| mask |
	mask := 0.
	aCollectionOfIndicies do:[:index| mask := mask bitOr: (1 bitShift: index)].
	self coreMask: (self coreMask bitOr:mask)! !

!Process methodsFor: '*RoarVM' stamp: 'dmu 5/5/2009 01:32'!
useOnlyCore: anIndex
	"Force this process to only run on this core (0-55)"
	self coreMask: (1 bitShift: anIndex)! !

!Process methodsFor: '*RoarVM' stamp: 'ssa 2/26/2009 03:02'!
useOnlyCores: aCollectionOfIndicies
	"Force this process to only run on these cores (0-55)"
	| mask |
	mask := 0.
	aCollectionOfIndicies do:[:index| mask := mask bitOr: (1 bitShift: index)].
	self coreMask: mask! !

!Process methodsFor: '*RoarVM' stamp: 'ads 1/30/2011 15:21'!
useOnlyMainCore
	RVMOperations isRVM ifTrue: [self useOnlyCore: (RVMMonitor getSample; mainRank)]! !

!SHPreferences class methodsFor: '*RoarVM' stamp: 'StefanMarr 3/18/2012 09:51'!
enabled
	SmalltalkImage current isRunningRoar ifTrue: [^false].
	^ enabled
		ifNil: [enabled := true]! !

!ProcessorScheduler methodsFor: '*RoarVM' stamp: 'StefanMarr 12/6/2011 10:05'!
thisProcess
	<primitive: 'primitiveThisProcess' > "must not specify module in order to hit obsolete_named_primitive_able and thus be local"
	^ self activeProcess! !

!ProcessorScheduler methodsFor: '*RoarVM' stamp: 'ads 1/30/2011 15:21'!
yieldIfFewerCoresThan: n
	RVMOperations coreCount < n    ifTrue: [self yield]! !

!Delay methodsFor: '*RoarVM' stamp: 'dmu 2/6/2011 21:25'!
spinWait
	| t |
	t := Time millisecondClockValue + delayDuration.
	[t > Time millisecondClockValue] whileTrue: [Processor yield]! !

!SmalltalkImage methodsFor: '*RoarVM-system attribute' stamp: 'StefanMarr 3/17/2012 20:21'!
isRunningOmni
	"Returns true if we're running on a Roar VM (0x828677 == RVM ascii code)"

	^[(self vmParameterAt: 0) = 16r798677] on: Error do:[:ex| ex return: false]! !

!SmalltalkImage methodsFor: '*RoarVM-system attribute' stamp: 'StefanMarr 3/17/2012 20:21'!
isRunningRoar
	"Returns true if we're running on a Roar VM (0x828677 == RVM ascii code)"

	^[(self vmParameterAt: 0) = 16r828677 or: [self isRunningOmni]] on: Error do:[:ex| ex return: false]! !

!SmalltalkImage methodsFor: '*RoarVM-snapshot and quit' stamp: 'StefanMarr 3/19/2012 14:45'!
setGCParameters
	"Adjust the VM's default GC parameters to avoid premature tenuring."
	self isRunningRoar ifFalse: [
		self vmParameterAt: 5 put: 4000.  "do an incremental GC after this many allocations"
		self vmParameterAt: 6 put: 2000.  "tenure when more than this many objects survive the GC"
		self setGCBiasToGrowGCLimit: 16*1024*1024.
		self setGCBiasToGrow: 1.
	].
! !

!BlockContext methodsFor: '*RoarVM' stamp: 'ads 1/30/2011 15:22'!
cyclesToRunMe
	| s delta correction result |
	s := RVMOperations cycleCounter.
	self value.
	delta :=  RVMOperations cycleCounter - s.
	^ delta < 0 ifTrue: [ 
		correction := SmallInteger maxVal + 1. 
		result := delta.
		[result < 0] whileTrue: [result := result + correction].
		result
	] ifFalse: [delta]! !

!BlockContext methodsFor: '*RoarVM' stamp: 'dmu 4/7/2009 02:38'!
forkOn: aCore
	"Create and schedule a Process running the code in the receiver."

	^ self newProcess resumeOn: aCore! !

!BlockContext methodsFor: '*RoarVM' stamp: 'ssa 4/8/2009 20:09'!
forkOn: aCore with: anObject
	"Create and schedule a Process running the code in the receiver."

	^ (self newProcessWith: anObject) resumeOn: aCore! !

!BlockContext methodsFor: '*RoarVM' stamp: 'ssa 4/8/2009 20:08'!
forkOn: aCore with: anObject at: aPriority
	"Create and schedule a Process running the code in the receiver."

	| proc |
	proc := self newProcessWith: anObject.
	proc priority: aPriority.
	^proc resumeOn: aCore! !

!Object methodsFor: '*RoarVM-converting' stamp: 'ssa 11/13/2008 10:34'!
asObjectCoordinate
	"Answer an RVMObjectCoordinate with my manycore coordinate details"
	
	^RVMObjectCoordinate for: self! !

!Object methodsFor: '*RoarVM-Sly support' stamp: 'dmu 3/18/2010 15:47'!
primitiveGetExtraPreheaderWord: anObject
	"ignores receiver, queries argument"
	<primitive: 'primitiveGetExtraPreheaderWord' module: 'RVMPlugin'>
	self primitiveFailed! !

!Object methodsFor: '*RoarVM-Sly support' stamp: 'dmu 3/18/2010 14:52'!
primitiveSetExtraPreheaderWord: w
	"sets word IN w to SELF"
	<primitive: 'primitiveSetExtraPreheaderWord' module: 'RVMPlugin'>
	self primitiveFailed! !

!Object methodsFor: '*RoarVM-Sly support' stamp: 'dmu 3/16/2010 19:11'!
primitiveSetExtraWordSelector: aSelector
	<primitive: 'primitiveSetExtraWordSelector' module: 'RVMPlugin'>
	self primitiveFailed! !

Object subclass: #RVMBarrier
	instanceVariableNames: 'interlock waitSema signalsNeededToPass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Misc'!

!RVMBarrier methodsFor: 'as yet unclassified' stamp: 'dmu 1/21/2010 18:49'!
initialize
	interlock := Semaphore new.
	interlock signal.
	waitSema := Semaphore new.
	signalsNeededToPass := 0! !

!RVMBarrier methodsFor: 'as yet unclassified' stamp: 'dmu 1/21/2010 19:14'!
signal
	interlock critical: [
		signalsNeededToPass := signalsNeededToPass - 1.
		signalsNeededToPass < 0 ifTrue: [self error: 'oops'].
		signalsNeededToPass = 0 ifTrue: [waitSema signal]
	]! !

!RVMBarrier methodsFor: 'as yet unclassified' stamp: 'dmu 1/21/2010 14:23'!
signalsNeededToPass: n
	signalsNeededToPass := n! !

!RVMBarrier methodsFor: 'as yet unclassified' stamp: 'dmu 1/21/2010 18:32'!
wait
	waitSema wait! !

!RVMBarrier methodsFor: 'as yet unclassified' stamp: 'dmu 1/21/2010 18:24'!
waitSema ^ waitSema! !

Object subclass: #RVMObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM'!

RVMObject subclass: #RVMAllCPUCoreStats
	instanceVariableNames: 'coreStats rawData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMAllCPUCoreStats class methodsFor: 'instance creation' stamp: 'ssa 8/19/2008 11:33'!
newFastFrom: data forMessageNames: messageNames
	"Answer a new instance of me using this data, which is a collection of arrays of core stats"
	| stats |
	stats := self new.
	stats initializeFastFrom: data forMessageNames: messageNames.
	^stats! !

!RVMAllCPUCoreStats class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:05'!
newFastestFrom: data forMessageNames: messageNames
	"Answer a new instance of me using this data, which is a collection of arrays of core stats"
	| stats |
	stats := self new.
	stats initializeFastestFrom: data forMessageNames: messageNames.
	^stats! !

!RVMAllCPUCoreStats class methodsFor: 'instance creation' stamp: 'ssa 8/13/2008 17:26'!
newFrom: data forMessageNames: messageNames
	"Answer a new instance of me using this data, which is a collection of arrays of core stats"
	| stats |
	stats := self new.
	stats initializeFrom: data forMessageNames: messageNames.
	^stats! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 08:39'!
allocationsSinceLastReadMostlys
	^self coreStats collect:[:each| each allocationsSinceLastReadMostly]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:53'!
allocationsSinceLastReadWrites
	^self coreStats collect:[:each| each allocationsSinceLastReadWrite]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:52'!
allocationsSinceLasts
	^self coreStats collect:[:each| each allocationsSinceLast]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:19'!
bytecodeCounts
	"RVMMulticoreSample capture bytecodeCounts"
	"Answer the collection of bytecode counts during this sample"
	
	^self coreStats collect:[:each| each bytecodesExecuted]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:37'!
bytecodesPerMillisecond
	"RVMMulticoreSample capture bytecodesPerMillisecond"
	"Answer the collection of ratios of bytecodes executed per millisecond during this sample"
	
	^self coreStats collect:[:each| each bytecodesPerMillisecond]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:24'!
bytecodesPerSecond
	"RVMMulticoreSample capture bytecodesPerSecond"
	"Answer the collection of ratios of bytecodes executed per second during this sample"
	
	^self coreStats collect:[:each| each bytecodesPerSecond]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 08:39'!
bytesLeftReadMostlys
	^self coreStats collect:[:each| each bytesLeftReadMostly]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:50'!
bytesLeftReadWrites
	^self coreStats collect:[:each| each bytesLeftReadWrite]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:49'!
bytesLefts
	^self coreStats collect:[:each| each bytesLeft]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 08:38'!
bytesUsedReadMostlys
	^self coreStats collect:[:each| each bytesUsedReadMostly]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:48'!
bytesUsedReadWrites
	^self coreStats collect:[:each| each bytesUsedReadWrite]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 1/1/1970 00:07'!
bytesUseds
	^self coreStats collect:[:each| each bytesUsed]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 08:39'!
compactionsSinceLastReadMostlys
	^self coreStats collect:[:each| each compactionsSinceLastReadMostly]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:54'!
compactionsSinceLastReadWrites
	^self coreStats collect:[:each| each compactionsSinceLastReadWrite]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:54'!
compactionsSinceLasts
	^self coreStats collect:[:each| each compactionsSinceLast]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:07'!
contextChangeCounts
	"RVMMulticoreSample capture contextChangeCounts"
	"Answer the collection of contextChanges during this sample"
	
	^self coreStats collect:[:each| each contextChangeCount]! !

!RVMAllCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 16:52'!
coreStats
	"Answer the value of coreStats"

	coreStats isNil ifTrue:[self coreStats: OrderedCollection new].
	^coreStats! !

!RVMAllCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 16:52'!
coreStats: anObject
	"Set the value of coreStats"

	coreStats := anObject! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:52'!
cyclesRunWaitRatios
	"RVMMulticoreSample capture cyclesRunWaitRatios"
	"Answer the collection of ratios during this sample"
	
	^self coreStats collect:[:each| each cyclesRunWaitRatio]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:48'!
cyclesRunnings
	"RVMMulticoreSample capture cyclesRunnings"
	"Answer the collection of cyclesRunning counts during this sample"
	
	^self coreStats collect:[:each| each cyclesRunning]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:41'!
cyclesSinceLasts
	"RVMMulticoreSample capture cyclesSinceLasts"
	"Answer the collection of cyclesSinceLasts counts during this sample"
	
	^self coreStats collect:[:each| each cyclesSinceLast]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:48'!
cyclesWaitings
	"RVMMulticoreSample capture cyclesWaitings"
	"Answer the collection of cyclesWaiting counts during this sample"
	
	^self coreStats collect:[:each| each cyclesWaiting]! !

!RVMAllCPUCoreStats methodsFor: 'DNU hacks' stamp: 'ssa 3/27/2009 17:26'!
doesNotUnderstand: aMessage
	"Hack DNU to install measurement mesages for tally aspects"
	
	| code tallyAspect |
	(aMessage selector first: 5) = 'sumOf' ifTrue:[
		tallyAspect := aMessage selector last: aMessage selector - 5.
		tallyAspect at: 1 put: tallyAspect first asLowercase.
	(RVMMulticoreSample coreTallyAspects includes: tallyAspect)
		ifFalse:[^super doesNotUnderstand: aMessage].
	code := aMessage selector,'
	"Answer the sum for this tally aspect from all cores"
	^self sendTallySumsDictionary at: #',tallyAspect.
	self class compile: code classified: 'tallying' notifying: nil.
	^self perform: aMessage selector].
	^super doesNotUnderstand: aMessage! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/5/2011 12:55'!
gcMilliseconds
	^ self coreStats collect: [:cs| cs gcMilliseconds]! !

!RVMAllCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 8/19/2008 11:34'!
initializeFastFrom: data forMessageNames: messageNames
	"data is an array of arrays, each element comprising stats from a single core"
	
	self rawData: data.
	data do:[:each| self coreStats add: (RVMCPUCoreStats newFastFromData: each forMessageNames: messageNames)]

! !

!RVMAllCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:05'!
initializeFastestFrom: data forMessageNames: messageNames
	"data is an array of arrays, each element comprising stats from a single core"
	
	self rawData: data.
	data do:[:each| self coreStats add: (RVMCPUCoreStats newFastestFromData: each forMessageNames: messageNames)]

! !

!RVMAllCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 8/13/2008 17:25'!
initializeFrom: data forMessageNames: messageNames
	"data is an array of arrays, each element comprising stats from a single core"
	
	self rawData: data.
	data do:[:each| self coreStats add: (RVMCPUCoreStats newFromData: each forMessageNames: messageNames)]

! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:06'!
interruptCheckCounts
	"RVMMulticoreSample capture yieldCounts"
	"Answer the collection of yields during this sample"
	
	^self coreStats collect:[:each| each interruptCheckCount]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:06'!
millisecondsRunnings
	"RVMMulticoreSample capture millisecondsRunnings"
	"Answer the collection of millisecondsRunning counts during this sample"
	
	^self coreStats collect:[:each| each millisecondsRunning]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:37'!
millisecondsWaitings
	"RVMMulticoreSample capture millisecondsWaitings"
	"Answer the collection of millisecondsWaiting counts during this sample"
	
	^self coreStats collect:[:each| each millisecondsWaiting]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:43'!
msRunWaitRatios
	"RVMMulticoreSample capture msRunWaitRatios"
	"Answer the collection of ratios during this sample"
	
	^self coreStats collect:[:each| each msRunWaitRatio]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:29'!
msSinceLasts
	"RVMMulticoreSample capture msSinceLasts"
	"Answer the collection of msSinceLasts counts during this sample"
	
	^self coreStats collect:[:each| each msSinceLast]! !

!RVMAllCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 16:52'!
rawData
	"Answer the value of rawData"

	rawData isNil ifTrue:[self rawData: nil].
	^rawData! !

!RVMAllCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:11'!
rawData: anObject
	"Set the value of rawData"

	rawData := anObject! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:03'!
receiveTallySum
	| tallies sum s |
	tallies := self coreStats
				collect: [:each | each receiveTally].
	sum := Array new: (tallies at: 1) rawData size.
	1
		to: sum size
		do: [:i | 
			s := 0.
			1
				to: tallies size
				do: [:j | s := s
								+ ((tallies at: j) rawData
										at: i)].
			sum at: i put: s].
	^ sum! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 3/27/2009 16:53'!
sendTallySums
	"Answer an array with the sum of all my send tallies in order from all my coreStats"
	| sums stats |
	sums := self coreStats first sendTallySums.
	2 to: self coreStats size do:[:index| 
		stats := self coreStats at: index.
		sums := sums + stats sendTallySums].
	^sums! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 3/27/2009 16:27'!
sendTallySumsDictionary
	"Answer a dictionary with the sums of all my send tallies from all my coreStats associated with their name"
	| dict keys values |
	dict := Dictionary new.
	keys := RVMCoreMessageTally tallyOrder.
	values := self sendTallySums.
	keys with: values do:[:key :value| dict at: key put: value].
	^dict! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 3/26/2009 09:55'!
sendTallySumsSorted
	"Answer a sorted collection with the sums of all my send tallies from all my coreStats associated with their name"
	^self sendTallySumsDictionary associations asSortedCollection:[:a :b| a value > b value]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 3/27/2009 17:06'!
sendTallySumsSortedReport
	"Answer a string to be used as a report of the sorted send tally sums"
	| report |
	report := 'Sorted Send Tallies for all cores',String crlf,'--------------------------',String crlf.
	self sendTallySumsSorted do:[:assoc| report := report, assoc value printString,'  ',assoc key,String crlf].
	^report! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:07'!
unforcedInterruptCheckCounts
	"RVMMulticoreSample capture yieldCounts"
	"Answer the collection of yields during this sample"
	
	^self coreStats collect:[:each| each unforcedInterruptCheckCount]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:17'!
yieldCounts
	"RVMMulticoreSample capture yieldCounts"
	"Answer the collection of yields during this sample"
	
	^self coreStats collect:[:each| each yieldCount]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:16'!
yieldFrequencies
	"RVMMulticoreSample capture yieldFrequencies"
	
	^self coreStats collect:[:each| each yieldFrequency]! !

!RVMAllCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:37'!
yieldsPerSecond
	"RVMMulticoreSample capture yieldsPerSecond"
	"Answer the collection of ratios of yields  per second during this sample"
	
	^self coreStats collect:[:each| each yieldsPerSecond]! !

RVMObject subclass: #RVMCPUCoreStats
	instanceVariableNames: 'rawData millisecondsSinceLastQuery cyclesSinceLastQuery sendTallies receiveTally memorySystemStats squeakInterpreterStats messageNames myX myY myRank objectTableStats'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMCPUCoreStats class methodsFor: 'instance creation' stamp: 'ssa 8/19/2008 10:57'!
newFastFromData: data forMessageNames: messageNames

	| stats |
	stats := self new.
	stats messageNames: messageNames.
	stats initializeFastFrom: data.
	^stats! !

!RVMCPUCoreStats class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:05'!
newFastestFromData: data forMessageNames: messageNames

	| stats |
	stats := self new.
	stats messageNames: messageNames.
	stats initializeFastestFrom: data.
	^stats! !

!RVMCPUCoreStats class methodsFor: 'instance creation' stamp: 'ssa 8/13/2008 17:49'!
newFromData: data forMessageNames: messageNames

	| stats |
	stats := self new.
	stats messageNames: messageNames.
	stats initializeFrom: data.
	^stats! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:52'!
allocationsSinceLast
	^self allocationsSinceLastReadWrite + self allocationsSinceLastReadMostly! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:56'!
allocationsSinceLastReadMostly
	^self memorySystemStats readMostlyHeapStats allocationsSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:56'!
allocationsSinceLastReadWrite
	^self memorySystemStats readWriteHeapStats allocationsSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:18'!
bytecodesExecuted
	"Answer the count of bytecodes executed during this sample"
	
	^self squeakInterpreterStats bytecodesExecuted! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:18'!
bytecodesPerMillisecond
	"Answer the ratio of bytecodes executed per ms during this sample"
	
	^(self squeakInterpreterStats bytecodesExectued / (1 max: self millisecondsSinceLastQuery) asFloat ) rounded! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:18'!
bytecodesPerSecond
	"Answer the ratio of bytecodes executed per second during this sample"
	self millisecondsSinceLastQuery = 0 ifTrue:[^0].
	
	^(self squeakInterpreterStats bytecodesExecuted / (1 max: self millisecondsSinceLastQuery) asFloat * 1000) rounded! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:49'!
bytesLeft
	^self bytesLeftReadWrite + self bytesLeftReadMostly! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:49'!
bytesLeftReadMostly
	^self memorySystemStats readMostlyHeapStats bytesLeft ! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:49'!
bytesLeftReadWrite
	^self memorySystemStats readWriteHeapStats bytesLeft ! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:45'!
bytesUsed
	^self bytesUsedReadWrite + self bytesUsedReadMostly! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:44'!
bytesUsedReadMostly
	^self memorySystemStats readMostlyHeapStats bytesUsed ! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 1/1/1970 00:54'!
bytesUsedReadWrite
	^self memorySystemStats readWriteHeapStats bytesUsed ! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:54'!
compactionsSinceLast
	^self compactionsSinceLastReadWrite + self compactionsSinceLastReadMostly! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:57'!
compactionsSinceLastReadMostly
	^self memorySystemStats readMostlyHeapStats compactionsSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:56'!
compactionsSinceLastReadWrite
	^self memorySystemStats readWriteHeapStats compactionsSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:08'!
contextChangeCount
	"Answer the count of yields  during this sample"
	
	^self squeakInterpreterStats contextChangeCount! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:53'!
cyclesRunWaitRatio
	"Answer the ratio between my cyclesRunning and cyclesWaiting"
	
	^self squeakInterpreterStats cyclesRunWaitRatio! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:46'!
cyclesRunning
	"Answer the count of cycles spent running during this sample"
	
	^self squeakInterpreterStats cyclesRunning! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:41'!
cyclesSinceLast
	"Answer the count of cycles since last sample"
	
	^self cyclesSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:07'!
cyclesSinceLastQuery
	"Answer the value of cyclesSinceLastQuery"

	cyclesSinceLastQuery isNil ifTrue:[self cyclesSinceLastQuery: 0].
	^cyclesSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:07'!
cyclesSinceLastQuery: anObject
	"Set the value of cyclesSinceLastQuery"

	cyclesSinceLastQuery := anObject! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:46'!
cyclesWaiting
	"Answer the count of cycles spect waiting during this sample"
	
	^self squeakInterpreterStats cyclesWaiting! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'dmu 1/5/2011 12:48'!
gcMilliseconds
	^ self memorySystemStats gcMilliseconds! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 8/19/2008 11:22'!
initializeCoreStatsFrom: data
	| valueMap objectMap target |
	valueMap := { 
				'x' -> #myX:.
				'y' -> #myY:.
				'rank' -> #myRank:.
				}.
	objectMap := {
				'sendTalliesByCore' -> #initializeSendTalliesFrom:.
				'receiveTallies' -> #initializeReceiveTalliesFrom:.			
				}.
	target := self.
				
	self processMap: valueMap into: target forData: data.
	self processMap: objectMap into: target forData: data.! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'dmu 1/1/1970 01:09'!
initializeFastCoreStatsFrom: data

	self myX: (data at: 2).
	self myY: (data at: 4).
	self myRank: (data at: 6).
	
	self initializeFastSendTalliesFrom: (data at: 8).
	self initializeFastReceiveTalliesFrom: (data at: 10).
	
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 11/16/2008 15:47'!
initializeFastFrom: data
	"This implementations assumes order dependency in the array for speed.  
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| target |
	self rawData: data.
	target := self.
	target millisecondsSinceLastQuery: (data at: 2).
	target cyclesSinceLastQuery: (data at: 4).
	target initializeFastCoreStatsFrom: (data at: 6).
	target initializeFastMemorySystemStatsFrom: (data at: 8).
	target initializeFastSqueakInterpreterStatsFrom: (data at: 10).
	target initializeFastObjectTableStatsFrom: (data at: 12).
! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:06'!
initializeFastMemorySystemStatsFrom: data
	"This implementations assumes order dependency in the array for speed.  
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| target |
	target := self memorySystemStats.
	target gcCount: (data at: 2).
	target gcMilliseconds: (data at: 4).
	target gcCycles: (data at: 6).
	self  initializeFastReadWriteHeapStatsFrom: (data at: 8).
	self initializeFastReadMostlyHeapStatsFrom: (data at: 10).
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 11/16/2008 15:53'!
initializeFastObjectTableStatsFrom: data
	"This implementations assumes order dependency in the array for speed.
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| target |
	target := self objectTableStats.
	target allocatedEntryCount: (data at: 2).
	target entryCount: (data at: 4).
	target allocationsSinceLastQuery: (data at: 6).
	target entriesFreedSinceLastQuery: (data at: 8).
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:54'!
initializeFastReadMostlyHeapStatsFrom: data
	"This implementations assumes order dependency in the array for speed.
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	|  target |
	target := self memorySystemStats readMostlyHeapStats.
	target bytesUsed: (data at: 2).
	target bytesLeft: (data at: 4).
	target allocationsSinceLastQuery: (data at: 4).
	target compactionsSinceLastQuery: (data at: 8).
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:54'!
initializeFastReadWriteHeapStatsFrom: data
	"This implementations assumes order dependency in the array for speed.
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	|  target |
	target := self memorySystemStats readWriteHeapStats.
	target bytesUsed: (data at: 2).
	target bytesLeft: (data at: 4).
	target allocationsSinceLastQuery: (data at: 4).
	target compactionsSinceLastQuery: (data at: 8).
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 8/19/2008 11:21'!
initializeFastReceiveTalliesFrom: data
	"data looks like #(395 0 0 1041 1 520 0 0 0 121 0 1 0 1 0 1 28741 10 1 0)
	number of elements based on number of messages tallied"
	
	self receiveTally: (RVMCoreMessageTally fastForData: data)! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 3/25/2009 14:19'!
initializeFastSendTalliesFrom: data
	"data USED TO looks like 
		#('send:=tallies' #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
		'send:=tallies' #(4 0 0 0 0 4 0 0 0 0 0 0 0 0 0 0 11 0 0 0) 
		... as many as we have cores being used)
		number of elements based on number of messages tallied"
	"NOW DATA LOOKS LIKE #(0 0 0 4290 0 0 0 0 0 4840 0 0 55 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 11 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 322 0 0)"
	
	"OLD CODE
	2 to: data size by: 2 do:[:index|
		self sendTallies add: (RVMCoreMessageTally fastForData: (data at: index))]"
	
	"NEW CODE"
	self sendTallies add: (RVMCoreMessageTally fastForData: data )! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 9/9/2008 22:40'!
initializeFastSqueakInterpreterStatsFrom: data
	"This implementations assumes order dependency in the array for speed.  
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	|  target |
	target := self squeakInterpreterStats.
	target bytecodesExecuted: (data at: 2).
	target yieldCount: (data at: 4).
"	target millisecondsRunning: (data at: 6).
	target millisecondsWaiting: (data at: 8)."
	target cyclesRunning: (data at: 6).
	target cyclesWaiting: (data at: 8).
	target contextChangeCount: (data at: 10).
	target interruptCheckCount: (data at: 12).
	target unforcedInterruptCheckCount: (data at: 14)
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:07'!
initializeFastestCoreStatsFrom: data
	"skip the tallies for speed"
	self myX: (data at: 2).
	self myY: (data at: 4).
	self myRank: (data at: 6).
	
"	self initializeFastSendTalliesFrom: (data at: 8).
	self initializeFastReceiveTalliesFrom: (data at: 10)."
	
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 11/16/2008 15:48'!
initializeFastestFrom: data
	"This implementations assumes order dependency in the array for speed.  
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| target |
	self rawData: data.
	target := self.
	target millisecondsSinceLastQuery: (data at: 2).
	target cyclesSinceLastQuery: (data at: 4).
	target initializeFastestCoreStatsFrom: (data at: 6).
	target initializeFastMemorySystemStatsFrom: (data at: 8).
	target initializeFastSqueakInterpreterStatsFrom: (data at: 10).
	target initializeFastObjectTableStatsFrom: (data at: 12).
	! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 11/16/2008 15:48'!
initializeFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap objectMap target |
	self rawData: data.
	valueMap := { 
				'millisecs' -> #millisecondsSinceLastQuery:.
				'cycles' -> #cyclesSinceLastQuery:.
				}.
	objectMap := {
				'coreStats' -> #initializeCoreStatsFrom:.
				'memorySystemStats' -> #initializeMemorySystemStatsFrom:.
				'interpreterStats' -> #initializeSqueakInterpreterStatsFrom:.
				'objectTableStats' -> #initializeFastObjectTableStatsFrom:.
				}.
	target := self.
				
	self processMap: valueMap into: target forData: data.
	self processMap: objectMap into: target forData: data.! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:04'!
initializeMemorySystemStatsFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap target |
	self rawData: data.
	valueMap := { 
				'gcCount' -> #gcCount:.
				'gcMilliseconds' -> #gcMilliseconds:.
				'gcCycles' -> #gcCycles:.
				'readWriteHeapStats' -> #initializeReadWriteHeapStatsFrom:.
				'readMostlyHeapStats' -> #initializeReadMostlyHeapStatsFrom:.
				}.

	target := self memorySystemStats.
				
	self processMap: valueMap into: target forData: data.
! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 11/16/2008 15:50'!
initializeObjectTableStatsFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap objectMap target |
	valueMap := { 
				'allocatedEntryCount[rank]' -> #allocatedEntryCount:.
				'entryCount[rank]' -> #entryCount:.
				'allocationsSinceLastQuery[rank]' -> #allocationsSinceLastQuery:.
				'entriesFreedSinceLastQuery[rank]' -> #entriesFreedSinceLastQuery:.
				}.
	objectMap := {}.
	target := self objectTableStats.
				
	self processMap: valueMap into: target forData: data.
	self processMap: objectMap into: target forData: data.! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:54'!
initializeReadMostlyHeapStatsFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap  target |
	valueMap := { 
				'bytesUsed()' -> #bytesUsed:.
				'bytesLeft()' -> #bytesLeft:.
				'allocationsSinceLastQuery' -> #allocationsSinceLastQuery:.
				'compactionsSinceLastQuery' -> #compactionsSinceLastQuery:.
				}.
	target := self memorySystemStats readMostlyHeapStats.
				
	self processMap: valueMap into: target forData: data.
! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:54'!
initializeReadWriteHeapStatsFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap  target |
	valueMap := { 
				'bytesUsed()' -> #bytesUsed:.
				'bytesLeft()' -> #bytesLeft:.
				'allocationsSinceLastQuery' -> #allocationsSinceLastQuery:.
				'compactionsSinceLastQuery' -> #compactionsSinceLastQuery:.
				}.
	target := self memorySystemStats readWriteHeapStats.
				
	self processMap: valueMap into: target forData: data.
! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 8/13/2008 18:21'!
initializeReceiveTalliesFrom: data
	"data looks like #(395 0 0 1041 1 520 0 0 0 121 0 1 0 1 0 1 28741 10 1 0)
	number of elements based on number of messages tallied"
	
	self receiveTally: (RVMCoreMessageTally forMessageNames: self messageNames andData: data)! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 8/19/2008 11:31'!
initializeSendTalliesFrom: data
	"data looks like 
		#('send:=tallies' #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
		'send:=tallies' #(4 0 0 0 0 4 0 0 0 0 0 0 0 0 0 0 11 0 0 0) 
		... as many as we have cores being used)
		number of elements based on number of messages tallied"
	
	| tally |
	data do:[:each|
		each = 'sendTallies' ifFalse:[
			tally := RVMCoreMessageTally forMessageNames: self messageNames andData: each.
			self sendTallies add: tally]]! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'ssa 9/9/2008 22:24'!
initializeSqueakInterpreterStatsFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap target |
	self rawData: data.
	valueMap := { 
				'bytecodesExecuted' -> #bytecodesExecuted:.
				'yieldCount' -> #yieldCount:.
				'contextChangeCount' -> #contextChangeCount:.
				"'millisecondsRunning' -> #millisecondsRunning:."
				"'millisecondsWaiting' -> #millisecondsWaiting:."
				'cyclesRunning' -> #cyclesRunning:.
				'cyclesWaiting' -> #cyclesWaiting:.
				'contextChangeCount' -> #contextChangeCount:.
				'interruptCheckCount' -> #interruptCheckCount:.
				'unforcedInterruptCheckCount' -> #unforcedInterruptCheckCount:.
				}.

	target := self squeakInterpreterStats.
				
	self processMap: valueMap into: target forData: data.
! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:06'!
interruptCheckCount
	"Answer the count of yields  during this sample"
	
	^self squeakInterpreterStats interruptCheckCount! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:22'!
memorySystemStats
	"Answer the value of memorySystemStats"

	memorySystemStats isNil ifTrue:[self memorySystemStats: RVMMemorySystemStats new].
	^memorySystemStats! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:07'!
memorySystemStats: anObject
	"Set the value of memorySystemStats"

	memorySystemStats := anObject! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:49'!
messageNames
	"Answer the value of messageNames"

	messageNames isNil ifTrue:[self messageNames: nil].
	^messageNames! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:49'!
messageNames: anObject
	"Set the value of messageNames"

	messageNames := anObject! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:34'!
millisecondsRunning
	"Answer the count of ms spent running during this sample"
	
	^self squeakInterpreterStats millisecondsRunning! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:08'!
millisecondsSinceLastQuery
	"Answer the value of millisecondsSinceLastQuery"

	millisecondsSinceLastQuery isNil ifTrue:[self millisecondsSinceLastQuery: 0].
	^millisecondsSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:07'!
millisecondsSinceLastQuery: anObject
	"Set the value of millisecondsSinceLastQuery"

	millisecondsSinceLastQuery := anObject! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:53'!
millisecondsWaiting
	"Answer the count of ms spect waiting during this sample"
	
	^self squeakInterpreterStats millisecondsWaiting! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:43'!
msRunWaitRatio
	"Answer the ratio between my msRunning and msWaiting"
	
	^self squeakInterpreterStats msRunWaitRatio! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:27'!
msSinceLast
	"Answer the count of ms since last sample"
	
	^self millisecondsSinceLastQuery! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:58'!
myRank
	"Answer the value of myRank"

	myRank isNil ifTrue:[self myRank: nil].
	^myRank! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:58'!
myRank: anObject
	"Set the value of myRank"

	myRank := anObject! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:58'!
myX
	"Answer the value of myX"

	myX isNil ifTrue:[self myX: nil].
	^myX! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:58'!
myX: anObject
	"Set the value of myX"

	myX := anObject! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:58'!
myY
	"Answer the value of myY"

	myY isNil ifTrue:[self myY: nil].
	^myY! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:58'!
myY: anObject
	"Set the value of myY"

	myY := anObject! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 11/16/2008 15:54'!
objectTableStats
	"Answer the value of objectTableStats"

	objectTableStats isNil ifTrue:[self objectTableStats: RVMObjectTableStats new].
	^objectTableStats! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 11/16/2008 15:54'!
objectTableStats: anObject
	"Set the value of objectTableStats"

	objectTableStats := anObject! !

!RVMCPUCoreStats methodsFor: 'initialize' stamp: 'dmu 8/15/2008 21:03'!
processMap: map into: targetObject forData: data
				
	| index datum |
	map do:[:assoc |
		index := data indexOf: assoc key.
		index = 0 ifTrue:[
			assoc key = 'receiveTallies' ifFalse:[self error:'Can''t find ',assoc key,' in sample data']].
		datum := data at: index + 1.
		targetObject perform: assoc value with: datum].
! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:49'!
rawData
	"Answer the value of rawData"

	rawData isNil ifTrue:[self rawData: nil].
	^rawData! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
rawData: anObject
	"Set the value of rawData"

	rawData := anObject! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:12'!
receiveTally
	"Answer the value of receiveTally"

	receiveTally isNil ifTrue:[self receiveTally: nil].
	^receiveTally! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:12'!
receiveTally: anObject
	"Set the value of receiveTally"

	receiveTally := anObject! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:20'!
sendTallies
	"Answer the value of sendTallies"

	sendTallies isNil ifTrue:[self sendTallies: OrderedCollection new].
	^sendTallies! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:07'!
sendTallies: anObject
	"Set the value of sendTallies"

	sendTallies := anObject! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 3/27/2009 16:55'!
sendTallySums
	"Answer an array with the sum of all my send tallies in order"
	| sums tally |
	self sendTallies isEmpty ifTrue:[^#()].
	sums := self sendTallies first talliesInOrder.
	2 to: self sendTallies size do:[:index| 
		tally := self sendTallies at: index.
		sums := sums + tally talliesInOrder].
	^sums! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:11'!
squeakInterpreterStats
	"Answer the value of squeakInterpreterStats"

	squeakInterpreterStats isNil ifTrue:[self squeakInterpreterStats: RVMSqueakInterpreterStats new].
	^squeakInterpreterStats! !

!RVMCPUCoreStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:07'!
squeakInterpreterStats: anObject
	"Set the value of squeakInterpreterStats"

	squeakInterpreterStats := anObject! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:06'!
unforcedInterruptCheckCount
	"Answer the count of yields  during this sample"
	
	^self squeakInterpreterStats unforcedInterruptCheckCount! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:16'!
yieldCount
	"Answer the count of yields  during this sample"
	
	^self squeakInterpreterStats yieldCount! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:17'!
yieldFrequency
	
	^self squeakInterpreterStats yieldFrequency! !

!RVMCPUCoreStats methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:19'!
yieldsPerSecond
	"Answer the ratio of yields per second during this sample"
	
	^(self squeakInterpreterStats yieldCount / (1 max: self millisecondsSinceLastQuery) asFloat * 1000) rounded! !

RVMObject subclass: #RVMCircularCollection
	instanceVariableNames: 'array begin end numberOfElements size nextIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Misc'!

!RVMCircularCollection class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:19'!
new: anInteger

	| collection |
	collection := self new.
	collection size: anInteger.
	^collection! !

!RVMCircularCollection class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:42'!
new: anInteger withAll: aCollection

	| collection |
	collection := self new: anInteger.
	aCollection do:[:each| collection add: each].
	^collection! !

!RVMCircularCollection class methodsFor: 'tests' stamp: 'ssa 1/1/1970 00:36'!
testAdd
	"RVMCircularCollection testAdd"
	
	| cc bag |
	cc := self new: 10.
	5 timesRepeat:[cc add: 1].
	bag := cc asOrderedCollection asBag.
	bag size = 5 & ((bag occurrencesOf: 1) = 5 ) ifTrue:['passed' print]ifFalse:['failed' print]! !

!RVMCircularCollection class methodsFor: 'tests' stamp: 'ssa 1/1/1970 00:39'!
testCollect
	"RVMCircularCollection testCollect"
	
	| cc results |
	cc := self new: 5.
	1 to: 20 do:[:each| cc add: each].
	results := cc collect:[:each|  each].
	results asArray = #(16 17 18 19 20)
		 ifTrue:['passed' print]ifFalse:['failed' print]! !

!RVMCircularCollection class methodsFor: 'tests' stamp: 'ssa 1/1/1970 00:38'!
testDo
	"RVMCircularCollection testDo"
	
	| cc total |
	cc := self new: 5.
	1 to: 20 do:[:each| cc add: each].
	total := 0.
	cc do:[:each| total := total + each].
	total = #(20 19 18 17 16) sum
		 ifTrue:['passed' print]ifFalse:['failed' print]! !

!RVMCircularCollection class methodsFor: 'tests' stamp: 'ssa 1/1/1970 00:54'!
testResize
	"RVMCircularCollection testResize"
	
	| cc |
	cc := self new: 5.
	1 to: 20 do:[:each| cc add: each].
	cc size: 2.
	cc asArray = #(16 17)
		 ifTrue:['passed' print]ifFalse:['failed' print]! !

!RVMCircularCollection class methodsFor: 'tests' stamp: 'ssa 1/1/1970 00:35'!
testRollover
	"RVMCircularCollection testRollover"
	
	| cc |
	cc := self new: 10.
	1 to: 20 do:[:each| cc add: each].
	cc asOrderedCollection asArray = #(11 12 13 14 15 16 17 18 19 20)
		 ifTrue:['passed' print]ifFalse:['failed' print]! !

!RVMCircularCollection class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:42'!
withAll: aCollection

	^self new: aCollection size withAll: aCollection! !

!RVMCircularCollection methodsFor: 'add/remove elements' stamp: 'ssa 1/1/1970 00:03'!
add: anObject
	"Add this object to my collection, increment the lastIndex, adjust if there is a rollover."
	self begin = 0 ifTrue:[ "I have no elements, add the first one"
		self begin:1.
		self end: 1]
	ifFalse:[ "I have elements, add one more and adjust if needed"
		self numberOfElements = self size
			ifTrue:["need to rollover the first element"
				self begin = self size "adjust begin and end"
					ifTrue:[self begin: 1.
						self end: self size]
					ifFalse:[self end: self begin.
						self begin: self begin + 1]]
			ifFalse:["no rollover needed"
				self end: (self end + 1 min: self size)]].
	self array at: self end put: anObject.
	self incrementElements
				
				! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 00:53'!
array
	"Answer the value of array"

	array isNil ifTrue:[self array: (Array new: self size)].
	^ array! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 00:38'!
array: anObject
	"Set the value of array"

	array := anObject! !

!RVMCircularCollection methodsFor: 'converting' stamp: 'ssa 1/1/1970 00:40'!
asArray

	^self asOrderedCollection asArray! !

!RVMCircularCollection methodsFor: 'converting' stamp: 'ssa 10/28/2008 19:41'!
asCircularCollection
^self! !

!RVMCircularCollection methodsFor: 'converting' stamp: 'ssa 1/1/1970 00:20'!
asOrderedCollection

	| oc |
	oc := OrderedCollection new.
	self do:[:each| oc add: each].
	^oc! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:14'!
at: index

	^self array at: (self realIndexFor: index)! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:15'!
at: index put: anObject

	self array at: (self realIndexFor: index) put: anObject! !

!RVMCircularCollection methodsFor: 'testing' stamp: 'ssa 5/28/2010 16:55'!
atEnd
	"Acting like an infinite stream"
	^false! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 10/18/2008 22:00'!
atRandom

	^self at: self numberOfElements atRandom! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:03'!
begin
	"Answer the value of begin"

	begin isNil ifTrue:[self begin: 0].
	^ begin! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:02'!
begin: anObject
	"Set the value of begin"

	begin := anObject! !

!RVMCircularCollection methodsFor: 'enumerating' stamp: 'ssa 1/1/1970 00:18'!
collect: aBlock
	"Iterate over my elements in order and answer the results"
	| results |
	results := self class new: self size.
	self do:[:each| results add: (aBlock value: each)].
	^results! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 00:04'!
decrementElements

	self numberOfElements: ((self numberOfElements - 1) max: 0)! !

!RVMCircularCollection methodsFor: 'enumerating' stamp: 'ssa 1/1/1970 00:13'!
do: aBlock
	"Iterate over my elements in order"
	1 to: self numberOfElements do:[:index|
		aBlock value:(self at: index)]! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:03'!
end
	"Answer the value of end"

	end isNil ifTrue:[self end: 0].
	^ end! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:02'!
end: anObject
	"Set the value of end"

	end := anObject! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 10/26/2008 15:27'!
first

	^self at:1! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:12'!
incrementElements

	self numberOfElements: ((self numberOfElements + 1) min: self size)! !

!RVMCircularCollection methodsFor: 'testing' stamp: 'ssa 1/1/1970 00:55'!
isEmpty

	^self numberOfElements = 0! !

!RVMCircularCollection methodsFor: 'testing' stamp: 'ssa 1/1/1970 00:55'!
isFull

	^self numberOfElements = self size! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 10/26/2008 15:28'!
last

	^self at:self numberOfElements! !

!RVMCircularCollection methodsFor: 'stream emulation' stamp: 'ssa 10/18/2008 21:31'!
next
	"Answer the next element in line, then advance the nextIndex"
	| item |
	self isEmpty ifTrue:[self error:'Empty Collection'].
	item := self at: self nextIndex.
	self nextIndex: self nextIndex + 1.
	^item! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 10/18/2008 21:29'!
nextIndex
	"Answer the value of nextIndex"

	nextIndex isNil ifTrue:[self nextIndex: 1].
	^ nextIndex! !

!RVMCircularCollection methodsFor: 'accessing' stamp: 'ssa 10/18/2008 21:40'!
nextIndex: anInteger
	"Set the value of nextIndex"
	anInteger > self size ifTrue:[^nextIndex := 1].
	nextIndex := anInteger! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:03'!
numberOfElements
	"Answer the value of numberOfElements"

	numberOfElements isNil ifTrue:[self numberOfElements: 0].
	^ numberOfElements! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 01:02'!
numberOfElements: anObject
	"Set the value of numberOfElements"

	numberOfElements := anObject! !

!RVMCircularCollection methodsFor: 'printing' stamp: 'ssa 10/3/2008 21:17'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' size:',self size printString,' numberOfElements:',self numberOfElements printString! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 10/18/2008 21:37'!
realIndexFor: index
	"Answer the loop adjusted index value"
	| realIndex |
	index >  self size ifTrue:[realIndex := index \\ self size]
			ifFalse:[realIndex := index].
	realIndex := realIndex - 1 + self begin.
	realIndex > self size ifTrue:[realIndex := realIndex - self size].
	^realIndex
! !

!RVMCircularCollection methodsFor: 'add/remove elements' stamp: 'ssa 1/1/1970 00:41'!
removeAll
	self begin: 0.
	self end: 0.
	self array:nil.
	self numberOfElements:0! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 00:52'!
size
	"Answer the value of size"

	size isNil ifTrue:[self size: 10].
	^ size! !

!RVMCircularCollection methodsFor: 'private' stamp: 'ssa 1/1/1970 00:04'!
size: aNumber
	"Set the value of size"

	| contents oldSize |
	size = aNumber ifTrue:[^self].
	self isEmpty 
		ifTrue:[size := aNumber.
			self array:nil; numberOfElements:0]
		ifFalse:[contents := self asOrderedCollection.
			oldSize := contents size.
			size := aNumber.
			self begin:0;end:0;numberOfElements:0.
			self array: (Array new: size).
			1 to: (size min: oldSize) do:[:index|
				self add: (contents at: index)]].
		! !

RVMObject subclass: #RVMCoreMessageTally
	instanceVariableNames: 'talliesInOrderCache allocateOrRecycleContextMessage allocateOrRecycleContextJustTestingMessage recycleContextIfPossibleMessage flushInterpreterCachesMessage flushMethodCacheMessage flushByMethodMessage flushSelectiveMessage addObjectFromSnapshotMessage updateWholeInterpreterMessage updateEnoughInterpreterToTransferControlMessage updateEnoughInterpreterToTransferControlJustTestingMessage updateEnoughInterpreterToTransferControlJustTestingResponse zapUnusedPortionOfHeapMessage zapUnusedPortionOfHeapResponse verifyInterpreterAndHeapMessage doAllRootsThereMessage doAllRootsThereResponse doAllRootsHereMessage hereIsARootResponse newValueForOopMessage noMoreRootsResponse scanCompactOrMakeFreeObjectsMessage scanCompactOrMakeFreeObjectsResponse returnContextMessage runPrimitiveMessage runPrimitiveJustTestingMessage runPrimitiveResponse loadFunctionFromPluginMessage loadFunctionFromPluginResponse broadcastInterpreterDoublewordMessage broadcastInterpreterWordMessage broadcastInterpreterBoolMessage sampleOneCoreMessage sampleOneCoreResponse initiateBarrierMessage respondToBarrierMessage aboutToWriteReadMostlyMemoryMessage aboutToWriteReadMostlyMemoryResponse wroteReadMostlyMemoryMessage flushFreeContextsMessage enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse rawData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMCoreMessageTally class methodsFor: 'code generation' stamp: 'ssa 3/26/2009 10:14'!
adaptToNewTallyMessages
	"RVMCoreMessageTally adaptToNewTallyMessages"
	"Change this class to work with the new messages"
	"BE SURE TO UPDATE tallyOrder MESSAGE IF THE TALLY ORDER CHANGES"
	
| messageNames code index |
messageNames := (RVMMulticoreSample primitiveCapture at: 8) asOrderedCollection.
messageNames add: 'rawData'.

RVMObject subclass: #RVMCoreMessageTally
	instanceVariableNames: messageNames asArray
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RVM-Monitoring'.

code := 'fastForData: data
	"Answer a new instance of me initialized to tally these messages"
	| tally |
	tally := self new.
	tally rawData: data.
	'.
index := 0.
messageNames remove: 'rawData'.
messageNames do:[:messageName| index := index + 1.
	code := code, 'tally ',messageName,': (data at: ',index printString,').
	'].
code := code,'^tally'.
RVMCoreMessageTally class compile: code classified:'instance creation' notifying:nil.

RVMCoreMessageTally removeCategory: 'accessing'.
messageNames do:[:selector|
	code := selector,'
	"Answer the tally for this message."
	^',selector.
	RVMCoreMessageTally  compile: code classified:'accessing' notifying:nil.	
	code := selector,': aValue
	"Answer the tally for this message."
	',selector,' := aValue'.
	RVMCoreMessageTally  compile: code classified:'accessing' notifying:nil].	
! !

!RVMCoreMessageTally class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:00'!
fastForData: data
	"Answer a new instance of me initialized to tally these messages"
	| tally |
	tally := self new.
	tally rawData: data.
	tally allocateOrRecycleContextMessage: (data at: 1).
	tally allocateOrRecycleContextJustTestingMessage: (data at: 2).
	tally recycleContextIfPossibleMessage: (data at: 3).
	tally flushInterpreterCachesMessage: (data at: 4).
	tally flushMethodCacheMessage: (data at: 5).
	tally flushByMethodMessage: (data at: 6).
	tally flushSelectiveMessage: (data at: 7).
	tally addObjectFromSnapshotMessage: (data at: 8).
	tally updateWholeInterpreterMessage: (data at: 9).
	tally updateEnoughInterpreterToTransferControlMessage: (data at: 10).
	tally updateEnoughInterpreterToTransferControlJustTestingMessage: (data at: 11).
	tally updateEnoughInterpreterToTransferControlJustTestingResponse: (data at: 12).
	tally zapUnusedPortionOfHeapMessage: (data at: 13).
	tally zapUnusedPortionOfHeapResponse: (data at: 14).
	tally verifyInterpreterAndHeapMessage: (data at: 15).
	tally doAllRootsThereMessage: (data at: 16).
	tally doAllRootsThereResponse: (data at: 17).
	tally doAllRootsHereMessage: (data at: 18).
	tally hereIsARootResponse: (data at: 19).
	tally newValueForOopMessage: (data at: 20).
	tally noMoreRootsResponse: (data at: 21).
	tally scanCompactOrMakeFreeObjectsMessage: (data at: 22).
	tally scanCompactOrMakeFreeObjectsResponse: (data at: 23).
	tally returnContextMessage: (data at: 24).
	tally runPrimitiveMessage: (data at: 25).
	tally runPrimitiveJustTestingMessage: (data at: 26).
	tally runPrimitiveResponse: (data at: 27).
	tally loadFunctionFromPluginMessage: (data at: 28).
	tally loadFunctionFromPluginResponse: (data at: 29).
	tally broadcastInterpreterDoublewordMessage: (data at: 30).
	tally broadcastInterpreterWordMessage: (data at: 31).
	tally broadcastInterpreterBoolMessage: (data at: 32).
	tally sampleOneCoreMessage: (data at: 33).
	tally sampleOneCoreResponse: (data at: 34).
	tally initiateBarrierMessage: (data at: 35).
	tally respondToBarrierMessage: (data at: 36).
	tally aboutToWriteReadMostlyMemoryMessage: (data at: 37).
	tally aboutToWriteReadMostlyMemoryResponse: (data at: 38).
	tally wroteReadMostlyMemoryMessage: (data at: 39).
	tally flushFreeContextsMessage: (data at: 40).
	tally enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage: (data at: 41).
	tally enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse: (data at: 42).
	tally enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage: (data at: 43).
	tally enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse: (data at: 44).
	tally enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage: (data at: 45).
	tally enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse: (data at: 46).
	^tally! !

!RVMCoreMessageTally class methodsFor: 'instance creation' stamp: 'dmu 1/1/1970 00:02'!
forMessageNames: aCollectionOfMessageNames andData: data
	"Answer a new instance of me initialized to tally these messages"
	| tally |
	tally := self new.
	tally rawData: data.
	1 to: data size do:[:index|
		tally perform:((aCollectionOfMessageNames at: index),':') asSymbol with:(data at: index)].
	^tally! !

!RVMCoreMessageTally class methodsFor: 'accessing' stamp: 'ssa 3/26/2009 10:15'!
tallyOrder
	"Answer an array of tally names in the order they appear in the sample"
	"RVMMulticoreSample primitiveCapture at: 8"
	^#(#allocateOrRecycleContextMessage
	#allocateOrRecycleContextJustTestingMessage
	#recycleContextIfPossibleMessage
	#flushInterpreterCachesMessage
	#flushMethodCacheMessage
	#flushByMethodMessage
	#flushSelectiveMessage
	#addObjectFromSnapshotMessage
	#updateWholeInterpreterMessage
	#updateEnoughInterpreterToTransferControlMessage
	#updateEnoughInterpreterToTransferControlJustTestingMessage
	#updateEnoughInterpreterToTransferControlJustTestingResponse
	#zapUnusedPortionOfHeapMessage
	#zapUnusedPortionOfHeapResponse
	#verifyInterpreterAndHeapMessage
	#doAllRootsThereMessage
	#doAllRootsThereResponse
	#doAllRootsHereMessage
	#hereIsARootResponse
	#newValueForOopMessage
	#noMoreRootsResponse
	#scanCompactOrMakeFreeObjectsMessage
	#scanCompactOrMakeFreeObjectsResponse
	#returnContextMessage
	#runPrimitiveMessage
	#runPrimitiveJustTestingMessage
	#runPrimitiveResponse
	#loadFunctionFromPluginMessage
	#loadFunctionFromPluginResponse
	#broadcastInterpreterDoublewordMessage
	#broadcastInterpreterWordMessage
	#broadcastInterpreterBoolMessage
	#sampleOneCoreMessage
	#sampleOneCoreResponse
	#initiateBarrierMessage
	#respondToBarrierMessage
	#aboutToWriteReadMostlyMemoryMessage
	#aboutToWriteReadMostlyMemoryResponse
	#wroteReadMostlyMemoryMessage
	#flushFreeContextsMessage
	#enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage
	#enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse
	#enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage
	#enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse
	#enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage
	#enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse)
! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
aboutToWriteReadMostlyMemoryMessage
	"Answer the tally for this message."
	^aboutToWriteReadMostlyMemoryMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
aboutToWriteReadMostlyMemoryMessage: aValue
	"Answer the tally for this message."
	aboutToWriteReadMostlyMemoryMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
aboutToWriteReadMostlyMemoryResponse
	"Answer the tally for this message."
	^aboutToWriteReadMostlyMemoryResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
aboutToWriteReadMostlyMemoryResponse: aValue
	"Answer the tally for this message."
	aboutToWriteReadMostlyMemoryResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
addObjectFromSnapshotMessage
	"Answer the tally for this message."
	^addObjectFromSnapshotMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
addObjectFromSnapshotMessage: aValue
	"Answer the tally for this message."
	addObjectFromSnapshotMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
allocateOrRecycleContextJustTestingMessage
	"Answer the tally for this message."
	^allocateOrRecycleContextJustTestingMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
allocateOrRecycleContextJustTestingMessage: aValue
	"Answer the tally for this message."
	allocateOrRecycleContextJustTestingMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
allocateOrRecycleContextMessage
	"Answer the tally for this message."
	^allocateOrRecycleContextMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
allocateOrRecycleContextMessage: aValue
	"Answer the tally for this message."
	allocateOrRecycleContextMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
broadcastInterpreterBoolMessage
	"Answer the tally for this message."
	^broadcastInterpreterBoolMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
broadcastInterpreterBoolMessage: aValue
	"Answer the tally for this message."
	broadcastInterpreterBoolMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
broadcastInterpreterDoublewordMessage
	"Answer the tally for this message."
	^broadcastInterpreterDoublewordMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
broadcastInterpreterDoublewordMessage: aValue
	"Answer the tally for this message."
	broadcastInterpreterDoublewordMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
broadcastInterpreterWordMessage
	"Answer the tally for this message."
	^broadcastInterpreterWordMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
broadcastInterpreterWordMessage: aValue
	"Answer the tally for this message."
	broadcastInterpreterWordMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 3/26/2009 10:05'!
determineTalliesInOrder
	"Answer an array of tallies in the order they appear in the sample"
	^self tallyOrder collect:[:label| self perform: label]! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
doAllRootsHereMessage
	"Answer the tally for this message."
	^doAllRootsHereMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
doAllRootsHereMessage: aValue
	"Answer the tally for this message."
	doAllRootsHereMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
doAllRootsThereMessage
	"Answer the tally for this message."
	^doAllRootsThereMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
doAllRootsThereMessage: aValue
	"Answer the tally for this message."
	doAllRootsThereMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
doAllRootsThereResponse
	"Answer the tally for this message."
	^doAllRootsThereResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
doAllRootsThereResponse: aValue
	"Answer the tally for this message."
	doAllRootsThereResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage
	"Answer the tally for this message."
	^enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage: aValue
	"Answer the tally for this message."
	enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse
	"Answer the tally for this message."
	^enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse: aValue
	"Answer the tally for this message."
	enforceCoherenceAfterEachCoreHasStoredIntoItsOwnHeapResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage
	"Answer the tally for this message."
	^enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage: aValue
	"Answer the tally for this message."
	enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse
	"Answer the tally for this message."
	^enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse: aValue
	"Answer the tally for this message."
	enforceCoherenceBeforeEachCoreStoresIntoItsOwnHeapResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage
	"Answer the tally for this message."
	^enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage: aValue
	"Answer the tally for this message."
	enforceCoherenceBeforeSenderStoresIntoAllHeapsMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse
	"Answer the tally for this message."
	^enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse: aValue
	"Answer the tally for this message."
	enforceCoherenceBeforeSenderStoresIntoAllHeapsResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushByMethodMessage
	"Answer the tally for this message."
	^flushByMethodMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushByMethodMessage: aValue
	"Answer the tally for this message."
	flushByMethodMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
flushFreeContextsMessage
	"Answer the tally for this message."
	^flushFreeContextsMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
flushFreeContextsMessage: aValue
	"Answer the tally for this message."
	flushFreeContextsMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushInterpreterCachesMessage
	"Answer the tally for this message."
	^flushInterpreterCachesMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushInterpreterCachesMessage: aValue
	"Answer the tally for this message."
	flushInterpreterCachesMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushMethodCacheMessage
	"Answer the tally for this message."
	^flushMethodCacheMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushMethodCacheMessage: aValue
	"Answer the tally for this message."
	flushMethodCacheMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushSelectiveMessage
	"Answer the tally for this message."
	^flushSelectiveMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
flushSelectiveMessage: aValue
	"Answer the tally for this message."
	flushSelectiveMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
hereIsARootResponse
	"Answer the tally for this message."
	^hereIsARootResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
hereIsARootResponse: aValue
	"Answer the tally for this message."
	hereIsARootResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
initiateBarrierMessage
	"Answer the tally for this message."
	^initiateBarrierMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
initiateBarrierMessage: aValue
	"Answer the tally for this message."
	initiateBarrierMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
loadFunctionFromPluginMessage
	"Answer the tally for this message."
	^loadFunctionFromPluginMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
loadFunctionFromPluginMessage: aValue
	"Answer the tally for this message."
	loadFunctionFromPluginMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
loadFunctionFromPluginResponse
	"Answer the tally for this message."
	^loadFunctionFromPluginResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
loadFunctionFromPluginResponse: aValue
	"Answer the tally for this message."
	loadFunctionFromPluginResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
newValueForOopMessage
	"Answer the tally for this message."
	^newValueForOopMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
newValueForOopMessage: aValue
	"Answer the tally for this message."
	newValueForOopMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
noMoreRootsResponse
	"Answer the tally for this message."
	^noMoreRootsResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
noMoreRootsResponse: aValue
	"Answer the tally for this message."
	noMoreRootsResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
rawData
	"Answer the tally for this message."
	^rawData! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
rawData: aValue
	"Answer the tally for this message."
	rawData := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
recycleContextIfPossibleMessage
	"Answer the tally for this message."
	^recycleContextIfPossibleMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
recycleContextIfPossibleMessage: aValue
	"Answer the tally for this message."
	recycleContextIfPossibleMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
respondToBarrierMessage
	"Answer the tally for this message."
	^respondToBarrierMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
respondToBarrierMessage: aValue
	"Answer the tally for this message."
	respondToBarrierMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
returnContextMessage
	"Answer the tally for this message."
	^returnContextMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
returnContextMessage: aValue
	"Answer the tally for this message."
	returnContextMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
runPrimitiveJustTestingMessage
	"Answer the tally for this message."
	^runPrimitiveJustTestingMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
runPrimitiveJustTestingMessage: aValue
	"Answer the tally for this message."
	runPrimitiveJustTestingMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
runPrimitiveMessage
	"Answer the tally for this message."
	^runPrimitiveMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
runPrimitiveMessage: aValue
	"Answer the tally for this message."
	runPrimitiveMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
runPrimitiveResponse
	"Answer the tally for this message."
	^runPrimitiveResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
runPrimitiveResponse: aValue
	"Answer the tally for this message."
	runPrimitiveResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
sampleOneCoreMessage
	"Answer the tally for this message."
	^sampleOneCoreMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
sampleOneCoreMessage: aValue
	"Answer the tally for this message."
	sampleOneCoreMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
sampleOneCoreResponse
	"Answer the tally for this message."
	^sampleOneCoreResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
sampleOneCoreResponse: aValue
	"Answer the tally for this message."
	sampleOneCoreResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
scanCompactOrMakeFreeObjectsMessage
	"Answer the tally for this message."
	^scanCompactOrMakeFreeObjectsMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
scanCompactOrMakeFreeObjectsMessage: aValue
	"Answer the tally for this message."
	scanCompactOrMakeFreeObjectsMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
scanCompactOrMakeFreeObjectsResponse
	"Answer the tally for this message."
	^scanCompactOrMakeFreeObjectsResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
scanCompactOrMakeFreeObjectsResponse: aValue
	"Answer the tally for this message."
	scanCompactOrMakeFreeObjectsResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 3/26/2009 10:05'!
talliesInOrder
	"Answer an array of tallies in the order they appear in the sample"
	^self talliesInOrderCache! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 3/26/2009 10:05'!
talliesInOrderCache
	"Answer the value of talliesInOrderCache"

	talliesInOrderCache isNil ifTrue:[self talliesInOrderCache: self determineTalliesInOrder].
	^ talliesInOrderCache! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 3/26/2009 10:04'!
talliesInOrderCache: anObject
	"Set the value of talliesInOrderCache"

	talliesInOrderCache := anObject! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 3/26/2009 10:16'!
tallyOrder
	"Answer an array of tally names in the order they appear in the sample"
	"RVMMulticoreSample primitiveCapture at: 8"
	^self class tallyOrder! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateEnoughInterpreterToTransferControlJustTestingMessage
	"Answer the tally for this message."
	^updateEnoughInterpreterToTransferControlJustTestingMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateEnoughInterpreterToTransferControlJustTestingMessage: aValue
	"Answer the tally for this message."
	updateEnoughInterpreterToTransferControlJustTestingMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateEnoughInterpreterToTransferControlJustTestingResponse
	"Answer the tally for this message."
	^updateEnoughInterpreterToTransferControlJustTestingResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateEnoughInterpreterToTransferControlJustTestingResponse: aValue
	"Answer the tally for this message."
	updateEnoughInterpreterToTransferControlJustTestingResponse := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateEnoughInterpreterToTransferControlMessage
	"Answer the tally for this message."
	^updateEnoughInterpreterToTransferControlMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateEnoughInterpreterToTransferControlMessage: aValue
	"Answer the tally for this message."
	updateEnoughInterpreterToTransferControlMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateWholeInterpreterMessage
	"Answer the tally for this message."
	^updateWholeInterpreterMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
updateWholeInterpreterMessage: aValue
	"Answer the tally for this message."
	updateWholeInterpreterMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
verifyInterpreterAndHeapMessage
	"Answer the tally for this message."
	^verifyInterpreterAndHeapMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
verifyInterpreterAndHeapMessage: aValue
	"Answer the tally for this message."
	verifyInterpreterAndHeapMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
wroteReadMostlyMemoryMessage
	"Answer the tally for this message."
	^wroteReadMostlyMemoryMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:23'!
wroteReadMostlyMemoryMessage: aValue
	"Answer the tally for this message."
	wroteReadMostlyMemoryMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
zapUnusedPortionOfHeapMessage
	"Answer the tally for this message."
	^zapUnusedPortionOfHeapMessage! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
zapUnusedPortionOfHeapMessage: aValue
	"Answer the tally for this message."
	zapUnusedPortionOfHeapMessage := aValue! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
zapUnusedPortionOfHeapResponse
	"Answer the tally for this message."
	^zapUnusedPortionOfHeapResponse! !

!RVMCoreMessageTally methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:22'!
zapUnusedPortionOfHeapResponse: aValue
	"Answer the tally for this message."
	zapUnusedPortionOfHeapResponse := aValue! !

RVMObject subclass: #RVMGroupStats
	instanceVariableNames: 'mainX mainY mainRank groupSize remaining width height'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMGroupStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:05'!
coreIndices
	"Answer a collection of points that give my core indicies as row@column"
	| points |
	points := OrderedCollection new.
	1 to: self height do:[:h|
		1 to: self width do:[:w|
			h*w-1 + w > self groupSize ifTrue:[^points  "handle odd last row case"].
			points add: w@h]].
	^points! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'ssa 11/13/2008 09:41'!
extent
	"Answer the dimensions of my tile grid"
	^self width @ self height! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:56'!
groupSize  ^ groupSize! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:56'!
groupSize: x groupSize := x! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:01'!
height ^ height! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:01'!
height: x height := x! !

!RVMGroupStats methodsFor: 'initialize-release' stamp: 'ssa 8/19/2008 10:56'!
initializeFastFrom: data

	self mainX:(data at: 2).
	self mainY: (data at: 4).
	self mainRank: (data at: 6).
	self width: (data at: 8).
	self height: (data at: 10).
	self groupSize: (data at: 12).
	self remaining: (data at: 14).
	! !

!RVMGroupStats methodsFor: 'initialize-release' stamp: 'dmu 8/15/2008 22:07'!
initializeFrom: data
1 to: data size by: 2 do: [:i|
	self perform: ((data at: i), ':') asSymbol with: (data at: i + 1)
	]! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:55'!
mainRank  ^ mainRank! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:57'!
mainRank: x mainRank := x! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:55'!
mainX  ^ mainX! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:57'!
mainX: x mainX := x! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:55'!
mainY  ^ mainY! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:57'!
mainY: y mainY := y! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:56'!
remaining  ^ remaining! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:57'!
remaining: r remaining := r! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:00'!
width ^ width! !

!RVMGroupStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:01'!
width: x width := x! !

RVMObject subclass: #RVMHeapStats
	instanceVariableNames: 'readWrite bytesUsed bytesLeft allocationsSinceLastQuery compactionsSinceLastQuery'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMHeapStats class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:27'!
newReadMostly

	^self new readWrite: false ! !

!RVMHeapStats class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:28'!
newReadWrite

	^self new readWrite: true! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:06'!
allocationsSinceLastQuery
	"Answer the number of object allocations since the last query"
	allocationsSinceLastQuery isNil ifTrue:[^0].
	^allocationsSinceLastQuery! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:07'!
allocationsSinceLastQuery: count
	"Set the number of object allocations since the last query"
	allocationsSinceLastQuery := count! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:04'!
bytesLeft
	"Answer the number of bytes left in this heap"
	bytesLeft isNil ifTrue:[^0].
	^bytesLeft! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:05'!
bytesLeft: count
	"Set the number of bytes left in this heap"
	bytesLeft := count! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 10/4/2008 08:38'!
bytesUsed
	"Answer the number of bytes used in this heap"
	bytesUsed isNil ifTrue:[^0].
	^bytesUsed ! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:04'!
bytesUsed: count
	"Set the number of bytes used in this heap"
	bytesUsed := count! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:06'!
compactionsSinceLastQuery
	"Answer the number of object compactions since the last query"
	compactionsSinceLastQuery isNil ifTrue:[^0].
	^compactionsSinceLastQuery! !

!RVMHeapStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:07'!
compactionsSinceLastQuery: count
	"Set the number of object compactions since the last query"
	compactionsSinceLastQuery := count! !

!RVMHeapStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:27'!
readWrite
	"Answer the value of readWrite"

	readWrite isNil ifTrue:[self readWrite: true].
	^ readWrite! !

!RVMHeapStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:27'!
readWrite: anObject
	"Set the value of readWrite"

	readWrite := anObject! !

RVMObject subclass: #RVMMemorySystemStats
	instanceVariableNames: 'gcCount gcMilliseconds gcCycles readWriteHeapStats readMostlyHeapStats'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:15'!
gcCount
	"Answer the value of gcCount"

	gcCount isNil ifTrue:[self gcCount: nil].
	^gcCount! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:15'!
gcCount: anObject
	"Set the value of gcCount"

	gcCount := anObject! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:15'!
gcCycles
	"Answer the value of gcCycles"

	gcCycles isNil ifTrue:[self gcCycles: nil].
	^gcCycles! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:15'!
gcCycles: anObject
	"Set the value of gcCycles"

	gcCycles := anObject! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:15'!
gcMilliseconds
	"Answer the value of gcMilliseconds"

	gcMilliseconds isNil ifTrue:[self gcMilliseconds: nil].
	^gcMilliseconds! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:15'!
gcMilliseconds: anObject
	"Set the value of gcMilliseconds"

	gcMilliseconds := anObject! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:52'!
readMostlyHeapStats
	"Answer the value of readMostlyHeapStats"

	readMostlyHeapStats isNil ifTrue:[self readMostlyHeapStats: RVMHeapStats newReadMostly].
	^ readMostlyHeapStats! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:51'!
readMostlyHeapStats: anObject
	"Set the value of readMostlyHeapStats"

	readMostlyHeapStats := anObject! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:51'!
readWriteHeapStats
	"Answer the value of readWriteHeapStats"

	readWriteHeapStats isNil ifTrue:[self readWriteHeapStats: RVMHeapStats newReadWrite].
	^ readWriteHeapStats! !

!RVMMemorySystemStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:51'!
readWriteHeapStats: anObject
	"Set the value of readWriteHeapStats"

	readWriteHeapStats := anObject! !

RVMObject subclass: #RVMMonitor
	instanceVariableNames: ''
	classVariableNames: 'DelayTiming LastSample Listeners RecordingFile RecordingMode SampleLimit Samples SamplingProcess ShowHeartbeat ShowTestAspects TurboSampling'
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMMonitor class methodsFor: 'autocoding' stamp: 'ssa 1/1/1970 00:03'!
autocodeScalerMethodFor: aSelector
	"Autocode and install a default scaler method for the aspect found in this selector."
	"Note this also adds a class variable of the form SCALERaspect"
	
	| aspect code var |
	aspect := aSelector allButLast:'Scaler' size.
	var := 'SCALER',aspect.
	self addClassVarName: var.
	code := aSelector,'
	"Answer the default scaler for ',aspect,'."
	',var,' isNil ifTrue:[',var,' := RVMDataScaler aspect: #',aspect,' min: 0 max: 100].
	^',var.
	self class compile: code classified:'scalers' notifying:nil! !

!RVMMonitor class methodsFor: 'autocoding' stamp: 'ssa 1/1/1970 00:03'!
autocodeScalerMethodForScaler: aDataScaler
	"Autocode and install a default scaler method for the aspect found in this selector."
	"Note this also adds a class variable of the form SCALERaspect"
	
	| aspect code var |
	aspect := aDataScaler aspect.
	var := 'SCALER',aspect.
	(self classVarNames includes: var) ifFalse:[self addClassVarName: var].
	code := aspect,'Scaler
	"Answer the default scaler for ',aspect,'."
	',var,' isNil ifTrue:[',var,' := RVMDataScaler aspect:#',aspect,' min: ',aDataScaler min printString,' max: ',aDataScaler max printString,'].
	^',var.
	self class compile: code classified:'scalers' notifying:nil! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 1/1/1970 00:12'!
clearSamples
	"Clear my collection of samples"
	Samples := nil.
	LastSample := nil.! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/19/2008 00:27'!
closeRecordingFile
	"Close the recording file"
	self recordingFile flush;close.
	self recordingFile: nil! !

!RVMMonitor class methodsFor: 'queries' stamp: 'ssa 1/1/1970 00:18'!
coreCount

^self lastSample  groupStats groupSize! !

!RVMMonitor class methodsFor: 'data generation' stamp: 'ssa 9/13/2008 19:40'!
dataFor: anAspect
	"Answer a generated data value for this aspect, based on the range of its scaler"
	
	^(self scalerFor:anAspect) atRandom! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 21:19'!
delaySamplingDuring: aBlock

	self sampling 
		ifTrue:[self stopSampling.
			aBlock value.
			self startSampling]
		ifFalse:[aBlock value]! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 20:25'!
delayTiming

	DelayTiming isNil ifTrue:[DelayTiming := #normal].
	^DelayTiming! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 20:25'!
delayTiming: aSymbol
	"#normal or #short"

	DelayTiming := aSymbol! !

!RVMMonitor class methodsFor: 'autocoding' stamp: 'ssa 1/1/1970 00:49'!
doesNotUnderstand: aMessage
	"autocode a method for aMessage if its one I recognize."
	(aMessage selector isUnary & (aMessage selector endsWith: 'Scaler')) 
		ifTrue:[self halt:'this should not be used anymore'.
			self  autocodeScalerMethodFor: aMessage selector.
			^self perform: aMessage selector].
		
	^super doesNotUnderstand: aMessage! !

!RVMMonitor class methodsFor: 'queries' stamp: 'ssa 11/27/2008 14:45'!
extent

^self lastSample  extent! !

!RVMMonitor class methodsFor: 'listeners' stamp: 'ssa 1/1/1970 00:07'!
forget: someone

	self listeners remove: someone ifAbsent:[]! !

!RVMMonitor class methodsFor: 'listeners' stamp: 'ssa 10/28/2008 19:54'!
forgetAll: someListeners

	someListeners do:[:each|
	self forget: each]! !

!RVMMonitor class methodsFor: 'listeners' stamp: 'ssa 1/1/1970 00:07'!
forgetListeners

	Listeners := nil! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ads 1/30/2011 15:21'!
getSample
	"Get the next sample"
	LastSample := self turboSampling ifTrue:[RVMMulticoreSample captureFastest]ifFalse:[RVMMulticoreSample next].
	RVMOperations isRVM & self isRecording ifTrue:[
				self recordSample: LastSample rawData].
! !

!RVMMonitor class methodsFor: 'listeners' stamp: 'ssa 8/25/2008 05:02'!
inform: someone

	self listeners add: someone! !

!RVMMonitor class methodsFor: 'listeners' stamp: 'ssa 2/18/2009 17:42'!
informListeners

	self listeners do:[:each| each hear: self]


"	| sema |
	sema := Semaphore  new.
	self listeners do:[:each| [each hear: self. sema signal] copy fixTemps fork].
	self listeners do: [:each| sema wait]"
	! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:41'!
isRecording

	^self recordingMode ~= #off! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 11/15/2008 23:42'!
lastSample
	"Answer the last recorded sample"
	LastSample isNil ifTrue:[self getSample].
	^LastSample ! !

!RVMMonitor class methodsFor: 'listeners' stamp: 'ssa 8/25/2008 04:59'!
listeners
	"Answer the list of listeners to me"
	Listeners isNil ifTrue:[Listeners := OrderedCollection new].
	^Listeners! !

!RVMMonitor class methodsFor: 'queries' stamp: 'ssa 11/17/2008 13:22'!
mainRank

^self lastSample  groupStats mainRank ! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 20:23'!
normalDelayTiming
	^self delayTiming = #normal! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:41'!
openRecordingFile
	"Open the recording file"
	self recordingFile ensureOpen! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:37'!
recordAllToFile: aCollection
	"Record each of these elements to the recording file in chunk format"
	
	aCollection do:[:each| self recordToFile: each]! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/28/2008 15:33'!
recordChunkToFile: anObject
	"Record the object on the recording file in chunk format"
	
	anObject printOn: self recordingFile.
	self recordingFile nextPut:$!!;cr;cr! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:45'!
recordSample: someData
	"Record this sample data based on the recording mode"
	
	self recordingMode = #off ifTrue:[^self].
	self recordingMode = #file ifTrue:[^self recordToFile: someData].
	self samples add: someData! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:24'!
recordingFile
	"Answer an appropriately named file for recording samples, open and ready."
	RecordingFile isNil ifTrue:[self recordingFile: self recordingFileFromUser].
	^RecordingFile! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/19/2008 00:28'!
recordingFile: aFile
	"Set an appropriately named file for recording samples, open and ready."
	RecordingFile isNil ifFalse:[RecordingFile close].
	RecordingFile := aFile! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:31'!
recordingFileFromUser
	"Answer an appropriately named file for recording samples, open and ready."
	^FileStream newFileNamed:'rvm-samples.recording'! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 23:19'!
recordingMode
	"Recording modes are #off, #loop, #file, and #long"
	RecordingMode isNil ifTrue:[RecordingMode := #loop].
	^RecordingMode! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/19/2008 00:52'!
recordingMode: nextMode
	"Recording modes are #off, #loop, #file, and #long"
	| lastMode |
	lastMode := RecordingMode.
	lastMode = nextMode ifTrue:[^self].
	self switchFrom: lastMode to: nextMode.
	RecordingMode := nextMode.
	self changed:#recordingMode! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/21/2008 11:56'!
sampleLimit

	SampleLimit isNil ifTrue:[SampleLimit := 2].
	^SampleLimit! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 8/22/2008 09:42'!
sampleLimit: anInteger

	SampleLimit := anInteger.
	self clearSamples! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/18/2008 22:14'!
samples
	"Answer my collection of samples"
	Samples isNil ifTrue:[self samples:(RVMCircularCollection new:self sampleLimit)].
	^Samples! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/18/2008 22:14'!
samples: aCollection
	"Set my collection of samples"
	Samples := aCollection! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 21:19'!
sampling
	"Am I current sampling?"
	^SamplingProcess notNil! !

!RVMMonitor class methodsFor: 'scaling' stamp: 'ssa 9/13/2008 19:39'!
scalersFor: aspects
	"Answer a collection of appropriate scalers for these aspects."

	^aspects collect:[:aspect|
		self scalerFor: aspect]! !

!RVMMonitor class methodsFor: 'heartbeat' stamp: 'ssa 4/1/2009 16:02'!
showHeartbeat

	ShowHeartbeat isNil ifTrue:[ShowHeartbeat := false].
	^ShowHeartbeat! !

!RVMMonitor class methodsFor: 'heartbeat' stamp: 'ssa 4/1/2009 16:03'!
showHeartbeat: aBoolean

	ShowHeartbeat := aBoolean! !

!RVMMonitor class methodsFor: 'test aspects' stamp: 'ssa 10/3/2008 15:06'!
showTestAspects

	ShowTestAspects isNil ifTrue:[ShowTestAspects := false].
	^ShowTestAspects! !

!RVMMonitor class methodsFor: 'test aspects' stamp: 'ssa 10/3/2008 15:06'!
showTestAspects: aBoolean

	ShowTestAspects := aBoolean! !

!RVMMonitor class methodsFor: 'heartbeat' stamp: 'ssa 4/3/2009 12:18'!
startHeartbeat
	"RVMMonitor startHeartbeat"
	"RVMMonitor stopHeartbeat"
	| rect delay |
	self showHeartbeat: true.
	rect := 0@0 extent: 4@4.
	delay := Delay forSeconds: 1.
	[[self showHeartbeat] whileTrue:[Display reverse: rect. delay wait]] forkAt: Processor highestPriority.
	Processor yield ! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ads 2/2/2011 14:52'!
startSampling
	"RVMMonitor startSampling"
	"Start the sampling process"
	| delay timeToRun delayTime lastTime|
	lastTime := Time now asSeconds.
	SamplingProcess notNil ifTrue:[self stopSampling].
	SamplingProcess := [[ Sensor leftShiftDown & Sensor controlKeyPressed]
								whileFalse: [
									"RVMOperations print: (Time now asSeconds - lastTime)  printString,' seconds since last sample'."
									 timeToRun := Time millisecondsToRun: [self getSample].
									lastTime := Time now asSeconds.
									"RVMOperations print:'sampling took ',(timeToRun/1000.0) printString,' seconds'."
									delayTime := self normalDelayTiming ifTrue:[10*timeToRun]ifFalse:[100 min: 10*timeToRun].
									delay := Delay forMilliseconds: delayTime.
									self informListeners.
									RVMMulticoreSample flush.  "clear sample buffers from monitor activity"
									delay wait]] newProcess.
	SamplingProcess name: 'RVMMonitor sampling process'.
	SamplingProcess resume	! !

!RVMMonitor class methodsFor: 'heartbeat' stamp: 'ssa 4/1/2009 16:09'!
stopHeartbeat
	"RVMMonitor startHeartbeat"
	"RVMMonitor stopHeartbeat"
	self showHeartbeat: false.! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 9/4/2008 15:00'!
stopSampling
	"RVMMonitor stopSampling"
	"Stop the sampling process"

	SamplingProcess isNil ifFalse:[SamplingProcess terminate].
	SamplingProcess := nil! !

!RVMMonitor class methodsFor: 'recording' stamp: 'ssa 10/18/2008 22:21'!
switchFrom: lastMode to: nextMode.
	"Switch recording modes.  Recording modes are #off, #loop, #file, and #long"
	nextMode = #off ifTrue:[
		lastMode = #loop ifTrue:["do nothing, leave collection in place"].
		lastMode = #long ifTrue:["do nothing, leave collection in place"].
		lastMode = #file ifTrue:[self closeRecordingFile]].
	nextMode = #loop ifTrue:[
		lastMode = #off ifTrue:[self samples: (RVMCircularCollection new:self sampleLimit)].
		lastMode = #long ifTrue:[self samples: (RVMCircularCollection withAll: self samples)].
		lastMode = #file ifTrue:[self closeRecordingFile]].
	nextMode = #long ifTrue:[
		lastMode = #off ifTrue:[self samples: OrderedCollection new].
		lastMode = #loop ifTrue:[self samples: self samples asOrderedCollection].
		lastMode = #file ifTrue:[self closeRecordingFile]].
	nextMode = #file ifTrue:[self openRecordingFile; recordAllToFile: self samples].
	
		! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 1/1/1970 00:16'!
turboOff
	"Turn off turboSampling"
	TurboSampling := false! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 1/1/1970 00:16'!
turboOn
	"Turn on turboSampling"
	TurboSampling := true! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 1/1/1970 00:16'!
turboSampling
	"Answer my  turboSampling flag"
	TurboSampling isNil ifTrue:[TurboSampling := true].
	^TurboSampling! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 21:22'!
useNormalDelay
	self delaySamplingDuring:[self delayTiming: #normal].
	! !

!RVMMonitor class methodsFor: 'sampling' stamp: 'ssa 10/7/2008 21:22'!
useShortDelay
	self delaySamplingDuring:[self delayTiming: #short].! !

RVMObject subclass: #RVMMulticoreSample
	instanceVariableNames: 'rawData messageNames cpuCoreStats groupStats runMask fence processInstanceCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMMulticoreSample commentStamp: 'ssa 8/19/2008 11:52' prior: 0!
two ways to capture a sample:
	RVMMulticoreSample captureSlow - uses a flexible but slower approach to handle data pairs
	            "            captureFast - fast, order dependent version (6x faster)
	            "            capture - calls captureFast

"performance test"
results := OrderedCollection new.
100 timesRepeat:[results add: (Time millisecondsToRun:[1000 timesRepeat:[RVMMulticoreSample captureFast]])/1000.0].
results average 

captureFast = 0.13813 
captureSlow = 0.80164

MessageTally spyOn:[1000 timesRepeat:[RVMMulticoreSample captureFast]]
shows a 6x improvement over captureSlow!

!RVMMulticoreSample class methodsFor: 'archiving' stamp: 'ssa 11/16/2008 15:07'!
archiveVersion
	^2! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 10/3/2008 15:10'!
aspects
	"Answer a list of message selectors that will return arrays of numbers for my monitor"
	^(RVMMulticoreSample allMethodsInCategory: #measuring),
	(RVMMonitor showTestAspects ifTrue:[self testAspects]ifFalse:[#()])! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 10/3/2008 22:51'!
aspectsUnder: aSymbol
	"Answer the aspects categorized under #system, #core, #testSystem, #testCore."
	aSymbol == #system ifTrue:[^self systemAspects].
	aSymbol == #core ifTrue:[^self coreAspects].
	aSymbol == #testSystem ifTrue:[^self testSystemAspects].
	aSymbol == #testCore ifTrue:[^self testCoreAspects].
	^self aspects! !

!RVMMulticoreSample class methodsFor: 'instance creation' stamp: 'ssa 8/19/2008 11:48'!
capture
	"Capture a sample from the vm and answer a new instance of me holding those values"
	"RVMMulticoreSample capture"
	
	^self captureFast! !

!RVMMulticoreSample class methodsFor: 'instance creation' stamp: 'ssa 8/19/2008 11:34'!
captureFast
	"Capture a sample from the vm and answer a new instance of me holding those values"
	"RVMMulticoreSample captureFast"
	| data sample |
	data := self primitiveCapture.
	sample := self new.
	sample initializeFastFrom: data.
	^sample! !

!RVMMulticoreSample class methodsFor: 'instance creation' stamp: 'ssa 1/1/1970 00:03'!
captureFastest
	"Capture a sample from the vm and answer a new instance of me holding those values"
	"RVMMulticoreSample captureFastest"
	| data sample |
	data := self primitiveCapture.
	sample := self new.
	sample initializeFastestFrom: data.
	^sample! !

!RVMMulticoreSample class methodsFor: 'instance creation' stamp: 'ssa 8/19/2008 11:48'!
captureSlow
	"Capture a sample from the vm and answer a new instance of me holding those values"
	"RVMMulticoreSample captureSlow"
	| data sample |
	data := self primitiveCapture.
	sample := self new.
	sample initializeFrom: data.
	^sample! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'dmu 4/2/2009 02:09'!
coreAspects
	"Answer a list of message selectors that will return collections of core numbers for my monitor"
	
	^#(#allocationsReadWrites #allocationsReadMostlys #allocationsTotal #bytecodeCounts #bytecodesPerCycles #bytecodesPerKCs #bytecodesPerMCs #bytecodesPerMilliseconds #bytecodesPerSecond #bytesLeftReadWrites #bytesLeftReadMostlys #bytesLefts #bytesUsedReadWrites #bytesUsedReadMostlys #bytesUseds #compactionsReadWrites #compactionsReadMostlys #compactionsTotal #contextChangeCounts #cyclesRunWaitRatios #cyclesRunnings #cyclesSinceLasts #cyclesWaitings #interruptCheckCounts #millisecondsRunnings #millisecondsWaitings #msRunWaitRatios #msSinceLasts  #unforcedInterruptCheckCounts #yieldCounts #yieldFrequencies #yieldsPerSecond #processCountsPerCore #runningProcessCountsPerCore)! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 3/26/2009 10:18'!
coreTallyAspects
	"Answer a list of message selectors that will return tallies from core message tallies"
	
	^RVMCoreMessageTally tallyOrder! !

!RVMMulticoreSample class methodsFor: 'archiving' stamp: 'ssa 11/16/2008 15:10'!
dearchive: versionNumber from: aStream
	versionNumber = self archiveVersion ifTrue:[^self dearchiveFrom: aStream].	
	self error:'Can''t handle this archive version'! !

!RVMMulticoreSample class methodsFor: 'examples' stamp: 'ssa 8/13/2008 15:13'!
example
	"RVMMulticoreSample example"
	self capture explore! !

!RVMMulticoreSample class methodsFor: 'examples' stamp: 'ssa 9/9/2008 19:50'!
examplePrimitive
	"RVMMulticoreSample examplePrimitive"
	self primitiveCapture inspect! !

!RVMMulticoreSample class methodsFor: 'sample access' stamp: 'ssa 11/12/2008 12:23'!
flush

	self primitiveFlush! !

!RVMMulticoreSample class methodsFor: 'instance creation' stamp: 'ssa 10/26/2008 20:26'!
newFrom: rawData
	"Answer a new instance of me holding these values"
	"RVMMulticoreSample newFrom: self primitiveCapture"
	|  sample |
	sample := self new.
	sample initializeFastFrom: rawData.
	^sample! !

!RVMMulticoreSample class methodsFor: 'sample access' stamp: 'ssa 1/1/1970 00:32'!
next

	^self capture! !

!RVMMulticoreSample class methodsFor: 'primitive access' stamp: 'ssa 10/18/2008 21:42'!
primitiveCapture
	"Grab some data from the VM and return it"
	<primitive: 'primitiveSampleRVM' module: 'RVMPlugin'>
	[^RVMMonitor samples next]value.
	self primitiveFailed! !

!RVMMulticoreSample class methodsFor: 'primitive access' stamp: 'dmu 4/10/2009 17:39'!
primitiveCapture: flags
	"Grab some data from the VM and return it"
	<primitive: 'primitiveSampleRVM' module: 'RVMPlugin'>
	self primitiveFailed! !

!RVMMulticoreSample class methodsFor: 'primitive access' stamp: 'dmu 4/10/2009 17:39'!
primitiveCaptureOneCore
	"self primitiveCaptureOneCore"
	^ self primitiveCapture: RVMSampleFlags everything - RVMSampleFlags allCores! !

!RVMMulticoreSample class methodsFor: 'primitive access' stamp: 'ssa 11/12/2008 12:23'!
primitiveFlush
	"Clear out the sample buffer in the vm by grabbing a sample"
	<primitive: 'primitiveSampleRVM' module: 'RVMPlugin'>
	! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 7/16/2010 15:21'!
systemAspects
	"Answer a list of message selectors that will return single system numbers for my monitor"
	
	^#(#numberOfRunningProcesses #processCount #coreProcessUtilization #bytecodeCountsAverage #bytecodesPerCycle #bytecodesPerCyclesAverage #bytecodesPerMillisecond #bytecodesPerMillisecondsAverage #bytecodesPerSecondAverage #cyclesRunWaitRatiosAverage #cyclesRunningsAverage #cyclesSinceLastsAverage #cyclesWaitingsAverage #msRunWaitRatiosAverage #msRunningsAverage #msSinceLastsAverage #msWaitingsAverage #yieldCountsAverage #yieldsPerSecondAverage )! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 10/3/2008 15:14'!
testAspects
	"Answer a list of message selectors that will return arrays of test numbers for my monitor"
	
	^self testCoreAspects, self testSystemAspects! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 10/3/2008 15:14'!
testCoreAspects
	"Answer a list of message selectors that will return arrays of test numbers for my monitor"
	
	^RVMMulticoreSample allMethodsInCategory: 'test measuring - core' asSymbol! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 10/3/2008 15:14'!
testSystemAspects
	"Answer a list of message selectors that will return arrays of test numbers for my monitor"
	
	^RVMMulticoreSample allMethodsInCategory: 'test measuring - system' asSymbol! !

!RVMMulticoreSample class methodsFor: 'aspect access' stamp: 'ssa 10/6/2008 18:46'!
updateSystemAndCoreAspectMethods
	"RVMMulticoreSample updateSystemAndCoreAspectMethods"
	"Determine the lists of single value system aspects and multivalue core aspects and update the access methods for them"
	| all singles groups code sample |
	all := RVMMulticoreSample allMethodsInCategory: #measuring.
	singles := OrderedCollection new.
	groups := OrderedCollection new.
	sample := RVMMulticoreSample capture.
	all do:[:msg| ((sample perform: msg) isKindOf:Number)ifTrue:[singles add: msg]ifFalse:[groups add: msg]].
	groups remove:#receiveTallySum ifAbsent:[].
	code := 'coreAspects
	"Answer a list of message selectors that will return collections of core numbers for my monitor"
	
	^', groups asArray storeString.
	self class compile: code classified:'aspect access' notifying:nil.
	code := 'systemAspects
	"Answer a list of message selectors that will return single system numbers for my monitor"
	
	^', singles asArray storeString.
	self class compile: code classified:'aspect access' notifying:nil.
	
! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:58'!
allocationsReadMostlys
	"RVMMulticoreSample capture allocationsReadMostlys"
	"Answer the collection of allocationsSinceLastReadMostlys"
	
	^self cpuCoreStats allocationsSinceLastReadMostlys! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:58'!
allocationsReadWrites
	"RVMMulticoreSample capture allocationsReadWrites"
	"Answer the collection of allocationsSinceLastReadWrites"
	
	^self cpuCoreStats allocationsSinceLastReadWrites! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:53'!
allocationsSinceLastReadMostlys
	"RVMMulticoreSample capture allocationsSinceLastReadMostlys"
	"Answer the collection of allocationsSinceLastReadMostlys"
	
	^self cpuCoreStats allocationsSinceLastReadMostlys! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:53'!
allocationsSinceLastReadWrites
	"RVMMulticoreSample capture allocationsSinceLastReadWrites"
	"Answer the collection of allocationsSinceLastReadWrites"
	
	^self cpuCoreStats allocationsSinceLastReadWrites! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 08:00'!
allocationsTotal
	"RVMMulticoreSample capture allocationsTotal"
	"Answer the collection of allocationsSinceLast"
	
	^self cpuCoreStats allocationsSinceLasts! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 9/14/2008 20:50'!
aspects
	"Answer a list of message selectors that will return arrays of numbers for my monitor"
	
	^self class aspects! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:40'!
bytecodeCounts
	"RVMMulticoreSample capture bytecodeCounts"
	"Answer the collection of bytecode counts during this sample"
	
	^self cpuCoreStats bytecodeCounts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:14'!
bytecodeCountsAverage
	"RVMMulticoreSample capture bytecodeCountsAverage"
	"Answer the average of bytecode counts during this sample"
	
	^self cpuCoreStats bytecodeCounts average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:19'!
bytecodesPerCycle
	"RVMMulticoreSample capture bytecodesPerCycle"

	| avg |
	avg := self cyclesSinceLastsAverage.
	^avg = 0 
		ifTrue:[0] 
		ifFalse:[self bytecodeCounts sum / (1 max: self cyclesSinceLastsAverage) asFloat]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:05'!
bytecodesPerCycles
	"RVMMulticoreSample capture bytecodesPerCycles"
	^self bytecodeCounts with: self cyclesRunnings 
		collect:[:bc :ms| ms = 0 ifTrue:[0] ifFalse:[bc/ms asFloat]]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/15/2008 16:06'!
bytecodesPerCyclesAverage
	"RVMMulticoreSample capture bytecodesPerCyclesAverage"
	^self bytecodesPerCycles average! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:06'!
bytecodesPerKCs
^ self bytecodesPerCycles collect: [:x| x * 1000]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:09'!
bytecodesPerMCs
^ self bytecodesPerCycles collect: [:x| x * 1000000]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:20'!
bytecodesPerMillisecond
	"RVMMulticoreSample capture bytecodesPerMillisecond"
	^self bytecodeCounts sum / (1 max: self msSinceLastsAverage) asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:20'!
bytecodesPerMilliseconds
	"RVMMulticoreSample capture bytecodesPerMilliseconds"
	^self bytecodeCounts with: self msSinceLasts collect:[:bc :ms| bc/(1 max: ms) asFloat]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:35'!
bytecodesPerMillisecondsAverage
	"RVMMulticoreSample capture bytecodesPerMillisecondsAverage"
	^self bytecodesPerMilliseconds average! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:25'!
bytecodesPerSecond
	"RVMMulticoreSample capture bytecodesPerSecond"
	"Answer the collection of ratios of bytecodes executed per second during this sample"
	
	^self cpuCoreStats bytecodesPerSecond! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:14'!
bytecodesPerSecondAverage
	"RVMMulticoreSample capture bytecodesPerSecondAverage"
	"Answer the average of ratios of bytecodes executed per second during this sample"
	
	^self cpuCoreStats bytecodesPerSecond average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:50'!
bytesLeftReadMostlys
	"RVMMulticoreSample capture bytesLeftReadMostlys"
	"Answer the collection of bytesLeftReadMostlys"
	
	^self cpuCoreStats bytesLeftReadMostlys! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:50'!
bytesLeftReadWrites
	"RVMMulticoreSample capture bytesLeftReadWrites"
	"Answer the collection of bytesLeftReadWrites"
	
	^self cpuCoreStats bytesLeftReadWrites! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:50'!
bytesLefts
	"RVMMulticoreSample capture bytesLefts"
	"Answer the collection of bytesLeft"
	
	^self cpuCoreStats bytesLefts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:48'!
bytesUsedReadMostlys
	"RVMMulticoreSample capture bytesUsedReadMostlys"
	"Answer the collection of bytesUsedReadMostlys"
	
	^self cpuCoreStats bytesUsedReadMostlys! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:47'!
bytesUsedReadWrites
	"RVMMulticoreSample capture bytesUsedReadWrites"
	"Answer the collection of bytesUsedReadWrites"
	
	^self cpuCoreStats bytesUsedReadWrites! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:24'!
bytesUseds
	"RVMMulticoreSample capture bytesUseds"
	"Answer the collection of bytesUsed"
	
	^self cpuCoreStats bytesUseds! !

!RVMMulticoreSample methodsFor: 'co vs inco' stamp: 'ads 2/2/2011 14:52'!
coinBytes
" RVMOperations moveAllToReadMostlyHeaps. RVMOperations printCoInBytesUsed. Smalltalk garbageCollect. RVMOperations printCoInBytesUsed. RVMMulticoreSample capture coinBytes"
	^self cpuCoreStats coreStats
		collect: [:stat | (stat memorySystemStats readWriteHeapStats bytesUsed @ stat memorySystemStats readMostlyHeapStats bytesUsed)]! !

!RVMMulticoreSample methodsFor: 'co vs inco' stamp: 'dmu 10/5/2008 00:26'!
coinBytesTotal
"Smalltalk garbageCollect. RVMMulticoreSample capture coinBytes"
	| s |
	s := 0 @ 0.
	self cpuCoreStats coreStats
		do: [:stat | s := s + (stat memorySystemStats readWriteHeapStats bytesUsed @ stat memorySystemStats readMostlyHeapStats bytesUsed)].
	^ s! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:59'!
compactionsReadMostlys
	"RVMMulticoreSample capture compactionsSinceLastReadMostlys"
	"Answer the collection of compactionsSinceLastReadMostlys"
	
	^self cpuCoreStats compactionsSinceLastReadMostlys! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 07:59'!
compactionsReadWrites
	"RVMMulticoreSample capture compactionsReadWrites"
	"Answer the collection of compactionsSinceLastReadWrites"
	
	^self cpuCoreStats compactionsSinceLastReadWrites! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 10/4/2008 08:00'!
compactionsTotal
	"RVMMulticoreSample capture compactionsTotal"
	"Answer the collection of compactionsSinceLast"
	
	^self cpuCoreStats compactionsSinceLasts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:06'!
contextChangeCounts
	"RVMMulticoreSample capture contextChangeCounts"
	"Answer the collection of yields during this sample"
	
	^self cpuCoreStats contextChangeCounts! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:06'!
coreIndices

	^self groupStats coreIndices! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ads 1/30/2011 15:23'!
coreProcessUtilization

	^RVMOperations coreProcessUtilization asFloat! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:33'!
cpuCoreStats
	"Answer the value of cpuCoreStats"

	^cpuCoreStats! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 8/13/2008 15:11'!
cpuCoreStats: anObject
	"Set the value of cpuCoreStats"

	cpuCoreStats := anObject! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:52'!
cyclesRunWaitRatios
	"RVMMulticoreSample capture cyclesRunWaitRatios"
	"Answer the collection of ratios during this sample"
	
	^self cpuCoreStats cyclesRunWaitRatios! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:14'!
cyclesRunWaitRatiosAverage
	"RVMMulticoreSample capture cyclesRunWaitRatiosAverage"
	"Answer the average of ratios during this sample"
	
	^self cpuCoreStats cyclesRunWaitRatios average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:49'!
cyclesRunnings
	"RVMMulticoreSample capture cyclesRunnings"
	"Answer the collection of cyclesRunning counts during this sample"
	
	^self cpuCoreStats cyclesRunnings! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:14'!
cyclesRunningsAverage
	"RVMMulticoreSample capture cyclesRunningsAverage"
	"Answer the average of cyclesRunning counts during this sample"
	
	^self cpuCoreStats cyclesRunnings average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/15/2008 16:30'!
cyclesSinceLasts
	"RVMMulticoreSample capture cyclesSinceLasts"
	"Answer the cyclesSinceLasts  during this sample"
	
	^self cpuCoreStats cyclesSinceLasts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/15/2008 16:08'!
cyclesSinceLastsAverage
	"RVMMulticoreSample capture cyclesSinceLastsAverage"
	"Answer the average cyclesSinceLasts  during this sample"
	
	^self cpuCoreStats cyclesSinceLasts average! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:49'!
cyclesWaitings
	"RVMMulticoreSample capture cyclesWaitings"
	"Answer the collection of cyclesWaiting counts during this sample"
	
	^self cpuCoreStats cyclesWaitings! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:14'!
cyclesWaitingsAverage
	"RVMMulticoreSample capture cyclesWaitingsAverage"
	"Answer the average of cyclesWaiting counts during this sample"
	
	^self cpuCoreStats cyclesWaitings average asFloat! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 11/13/2008 09:42'!
extent
	"Answer the dimensions of my tile grid"
	^self groupStats extent! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 11/16/2008 15:03'!
fence
	"Answer the value of fence"

	fence isNil ifTrue:[self fence: nil].
	^ fence! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 11/16/2008 15:03'!
fence: anObject
	"Set the value of fence"

	fence := anObject! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/5/2011 12:55'!
gcMilliseconds
	"RVMMulticoreSample capture gcMilliseconds"
	^ self cpuCoreStats gcMilliseconds! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'dmu 8/15/2008 22:13'!
groupStats ^  groupStats! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'dmu 8/15/2008 22:14'!
groupStats: x    groupStats  := x! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 8/13/2008 17:32'!
initializeAllCPUCoreStatsFrom: data
	
	self cpuCoreStats: (RVMAllCPUCoreStats newFrom: data forMessageNames: self messageNames)! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 8/19/2008 10:50'!
initializeFastAllCPUCoreStatsFrom: data
	
	self cpuCoreStats: (RVMAllCPUCoreStats newFastFrom: data forMessageNames: self messageNames)! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 7/16/2010 16:45'!
initializeFastFrom: data
	"This implementations assumes order dependency in the array for speed.
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.    Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	self rawData: data.
	self runMask: (data at: 2).
	self messageNames: (data at: 4).
	self initializeFastGroupStatsFrom: (data at: 6).
	self initializeFastAllCPUCoreStatsFrom: (data at: 8).
	self fence: (data at: 10).
	self processInstanceCount: Process instanceCount
! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 3/25/2009 14:11'!
initializeFastGroupStatsFrom: data
	
	self groupStats: (RVMGroupStats new initializeFastFrom: data last)! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 1/1/1970 00:04'!
initializeFastestAllCPUCoreStatsFrom: data
	
	self cpuCoreStats: (RVMAllCPUCoreStats newFastestFrom: data forMessageNames: self messageNames)! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'dmu 5/21/2010 23:05'!
initializeFastestFrom: data
	"This implementations assumes order dependency in the array for speed.
	
	Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.    Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	self rawData: data.
	self runMask: (data at: 2).
	self messageNames: (data at: 4).
	self initializeFastestGroupStatsFrom: ((data at: 6 ) at: 2).
	self initializeFastestAllCPUCoreStatsFrom: (data at: 8).
	self fence: (data at: 10).
! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 3/25/2009 19:17'!
initializeFastestGroupStatsFrom: data
	
	self groupStats: (RVMGroupStats new initializeFastFrom: data )! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 7/16/2010 16:46'!
initializeFrom: data
	"Set my variables based on the data array.  The data array is organized as a series of name-value pairs, where the names are strings and the values are either numbers, strings, symbols or an array of name-value pairs.  This implementations assumes no order dependency in the array except that values always follow their names.  Subarrays will be passed to another method like me that will factor them into appropriate subobjects."
	
	| valueMap objectMap |
	self rawData: data.
	valueMap := { }.
	objectMap := { 
				'runMask' -> #runMask:.
				'messageNames' -> #messageNames:.
				'allCPUCoreStats' -> #initializeAllCPUCoreStatsFrom:.
				'groupStats' -> #initializeGroupStatsFrom:.
				'fence' -> #fence:.
				}.
				
	self processMap: valueMap into: self forData: data.
	self processMap: objectMap into: self forData: data.
	self processInstanceCount: Process instanceCount
! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'dmu 8/15/2008 22:11'!
initializeGroupStatsFrom: data
	
groupStats := RVMGroupStats new initializeFrom: data! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:08'!
interruptCheckCounts
	"RVMMulticoreSample capture yieldCounts"
	"Answer the collection of yields during this sample"
	
	^self cpuCoreStats interruptCheckCounts! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 8/13/2008 17:33'!
messageNames
	"Answer the value of messageNames"

	^messageNames! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 8/13/2008 15:11'!
messageNames: anObject
	"Set the value of messageNames"

	messageNames := anObject! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 8/17/2008 11:01'!
millisecondsRunnings
	"RVMMulticoreSample capture millisecondsRunnings"
	"Answer the collection of msRunning counts during this sample"
	
	^self cpuCoreStats millisecondsRunnings! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:41'!
millisecondsWaitings
	"RVMMulticoreSample capture millisecondsWaitings"
	"Answer the collection of millisecondsWaitings counts during this sample"
	
	^self cpuCoreStats millisecondsWaitings! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:44'!
msRunWaitRatios
	"RVMMulticoreSample capture msRunWaitRatios"
	"Answer the collection of ratios during this sample"
	
	^self cpuCoreStats msRunWaitRatios! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:15'!
msRunWaitRatiosAverage
	"RVMMulticoreSample capture msRunWaitRatiosAverage"
	"Answer the average of ratios during this sample"
	
	^self cpuCoreStats msRunWaitRatios average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:29'!
msRunningsAverage
	"RVMMulticoreSample capture msRunningsAverage"
	"Answer the average of millisecondsRunning counts during this sample"
	
	^self cpuCoreStats millisecondsRunnings average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:30'!
msSinceLasts
	"RVMMulticoreSample capture msSinceLasts"
	"Answer the collection of msSinceLasts  during this sample"
	
	^self cpuCoreStats msSinceLasts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/15/2008 15:30'!
msSinceLastsAverage
	"RVMMulticoreSample capture msSinceLastsAverage"
	"Answer the average msSinceLasts  during this sample"
	
	^self cpuCoreStats msSinceLasts average! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:43'!
msWaitingsAverage
	"RVMMulticoreSample capture msWaitingsAverage"
	"Answer the average of msWaiting counts during this sample"
	
	^self cpuCoreStats millisecondsWaitings average asFloat! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 1/1/1970 01:07'!
numberOfCores

	^self groupStats isNil ifTrue:[56]ifFalse:[self groupStats groupSize]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ads 1/30/2011 15:23'!
numberOfRunningProcesses

	^RVMOperations numberOfRunningProcesses ! !

!RVMMulticoreSample methodsFor: 'printing' stamp: 'ssa 8/15/2008 16:28'!
printOn: aStream

	aStream 
		nextPutAll: 'anRVMMulticoreStats(';
		cr;tab;nextPutAll: ' bytecodesPerCycle=';nextPutAll: self bytecodesPerCycle printString;
		cr;tab;nextPutAll: ' bytecodesPerCyclesAverage=';nextPutAll: self bytecodesPerCyclesAverage printString;
		cr;tab;nextPutAll: ' bytecodesPerMillisecond=';nextPutAll: self bytecodesPerMillisecond printString;
		cr;tab;nextPutAll: ' bytecodesPerMillisecondsAverage=';nextPutAll: self bytecodesPerMillisecondsAverage printString;
		cr;tab;nextPutAll: ' bytecodeCountsAvg=';nextPutAll: self bytecodeCountsAverage printString;
		cr;tab;nextPutAll: ' bytecodePerSecondAvg=';nextPutAll: self bytecodesPerSecondAverage printString;
		cr;tab;nextPutAll: ' cyclesRunningsAvg=';nextPutAll: self cyclesRunningsAverage printString;
		cr;tab;nextPutAll: ' cyclesWaitingsAvg=';nextPutAll: self cyclesWaitingsAverage printString;
		cr;tab;nextPutAll: ' cyclesRunWaitRatiosAvg=';nextPutAll: self cyclesRunWaitRatiosAverage printString;
		cr;tab;nextPutAll: ' msRunningsAvg=';nextPutAll: self msRunningsAverage printString;
		cr;tab;nextPutAll: ' msWaitingsAvg=';nextPutAll: self msWaitingsAverage printString;
		cr;tab;nextPutAll: ' msRunWaitRatiosAvg=';nextPutAll: self msRunWaitRatiosAverage printString;
		cr;tab;nextPutAll: ' yieldCountsAvg=';nextPutAll: self yieldCountsAverage printString;
		cr;tab;nextPutAll: ' yieldsPerSecondAvg=';nextPutAll: self yieldsPerSecondAverage printString;
		cr;tab;nextPutAll: ')';cr
		! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 7/16/2010 15:21'!
processCount
	^self processInstanceCount! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ads 1/30/2011 15:22'!
processCountsPerCore

 | bag counts |
bag := (Process allInstances asOrderedCollection  collect:[:obj| RVMOperations isRVM ifFalse:[RVMMonitor coreCount atRandom - 1]ifTrue:[RVMOperations getCoreHolding:obj]]) asBag.
counts := Array new: RVMMonitor coreCount.
1 to: RVMMonitor coreCount do:[:coreIndex| counts at: coreIndex put: (bag occurrencesOf: coreIndex)].
^counts
! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 7/16/2010 15:03'!
processInstanceCount
	"Answer the value of processInstanceCount"

	processInstanceCount isNil ifTrue:[self processInstanceCount: Process instanceCount].
	^ processInstanceCount! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 7/16/2010 15:02'!
processInstanceCount: anObject
	"Set the value of processInstanceCount"

	processInstanceCount := anObject! !

!RVMMulticoreSample methodsFor: 'initialize' stamp: 'ssa 8/13/2008 15:47'!
processMap: map into: targetObject forData: data
				
	| index datum |
	map do:[:assoc |
		index := data indexOf: assoc key.
		index = 0 ifTrue:[self error:'Can''t find ',assoc key,' in sample data'].
		datum := data at: index + 1.
		targetObject perform: assoc value with: datum].
! !

!RVMMulticoreSample methodsFor: 'test measuring - system' stamp: 'ssa 10/3/2008 15:01'!
random1
	"RVMMulticoreSample new random1"
	"generated data for testing visualization"
	
	^1 atRandom! !

!RVMMulticoreSample methodsFor: 'test measuring - system' stamp: 'ssa 10/3/2008 15:01'!
random10
	"RVMMulticoreSample new random10"
	"generated data for testing visualization"
	
	^10 atRandom! !

!RVMMulticoreSample methodsFor: 'test measuring - system' stamp: 'ssa 10/3/2008 15:01'!
random100
	"RVMMulticoreSample new random100"
	"generated data for testing visualization"
	
	^100 atRandom! !

!RVMMulticoreSample methodsFor: 'test measuring - system' stamp: 'ssa 10/3/2008 15:00'!
random1000
	"RVMMulticoreSample new random1000"
	"generated data for testing visualization"
	
	^1000 atRandom! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 8/13/2008 15:11'!
rawData
	"Answer the value of rawData"

	rawData isNil ifTrue:[self rawData: nil].
	^rawData! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'ssa 10/18/2008 18:03'!
rawData: anObject
	"Set the value of rawData"

	rawData := anObject! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:20'!
receiveTallySum
^ self cpuCoreStats receiveTallySum! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:04'!
runMask
^ runMask! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:05'!
runMask: x
  runMask := x! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ads 1/30/2011 15:23'!
runningProcessCountsPerCore


^RVMOperations runningProcessByCore  collect:[:obj| obj isNil ifTrue:[0]  ifFalse:[1]]! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 3/27/2009 17:04'!
sendTallySumsSortedReport
	"Answer a string report of the sorted send tallies across all cores"
	^self cpuCoreStats sendTallySumsSortedReport! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:28'!
test0001To1
	"RVMMulticoreSample new test0001To1"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0001.
	end := 1.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:28'!
test0to1
	"RVMMulticoreSample new test0to1"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0.
	end := 1.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:28'!
test0to10
	"RVMMulticoreSample new test0to10"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0.
	end := 10.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:32'!
test0to100
	"RVMMulticoreSample new test0to100"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0.
	end := 100.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^(testData copyFrom: 1 to: self numberOfCores) ! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:27'!
test0to1B
	"RVMMulticoreSample new test0to1B"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0.
	end := 1000000000.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:28'!
test0to1K
	"RVMMulticoreSample new test0to1K"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0.
	end := 1000.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:28'!
test0to1M
	"RVMMulticoreSample new test0to1M"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := 0.0.
	end := 1000000.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 10/7/2008 17:28'!
testNeg1To1
	"RVMMulticoreSample new testNeg1To1"
	"generated data for testing visualization"
	
	| testData start end |
	testData := OrderedCollection new.
	start := -1.0.
	end := 1.0.
	start to: end by: end - start / self numberOfCores do:[:value|
		testData add: value].
	^testData copyFrom: 1 to: self numberOfCores! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:07'!
testRandom1
	"RVMMulticoreSample new testRandom1"
	"generated data for testing visualization"
	
	| testData |
	testData := OrderedCollection new.
	self numberOfCores timesRepeat:[
		testData add: 1 atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:06'!
testRandom10
	"RVMMulticoreSample new testRandom10"
	"generated data for testing visualization"
	
	| testData |
	testData := OrderedCollection new.
	self numberOfCores timesRepeat:[
		testData add: 10 atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:06'!
testRandom100
	"RVMMulticoreSample new testRandom100"
	"generated data for testing visualization"
	
	| testData |
	testData := OrderedCollection new.
	self numberOfCores timesRepeat:[
		testData add: 100 atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:06'!
testRandom1000
	"RVMMulticoreSample new testRandom1000"
	"generated data for testing visualization"
	
	| testData |
	testData := OrderedCollection new.
	self numberOfCores timesRepeat:[
		testData add: 1000 atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:05'!
testRandom100s
	"RVMMulticoreSample new testRandom100s"
	"generated data for testing visualization"
	
	| testData min max delta |
	testData := OrderedCollection new.
	min := 100.
	max := 999.
	delta := max - min.
	self numberOfCores timesRepeat:[
		testData add: min + delta atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:05'!
testRandom1Ks
	"RVMMulticoreSample new testRandom1Ks"
	"generated data for testing visualization"
	
	| testData min max delta |
	testData := OrderedCollection new.
	min := 1000.
	max := 1999.
	delta := max - min.
	self numberOfCores timesRepeat:[
		testData add: min + delta atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:04'!
testRandomBs
	"RVMMulticoreSample new testRandomBs"
	"generated data for testing visualization"
	
	| testData min max delta |
	testData := OrderedCollection new.
	min := 1000000000.
	max := 999000000000.
	delta := max - min.
	self numberOfCores timesRepeat:[
		testData add: min + delta atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:05'!
testRandomKs
	"RVMMulticoreSample new testRandomKs"
	"generated data for testing visualization"
	
	| testData min max delta |
	testData := OrderedCollection new.
	min := 1000.
	max := 999000.
	delta := max - min.
	self numberOfCores timesRepeat:[
		testData add: min + delta atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'test measuring - core' stamp: 'ssa 1/1/1970 00:04'!
testRandomMs
	"RVMMulticoreSample new testRandomMs"
	"generated data for testing visualization"
	
	| testData min max delta |
	testData := OrderedCollection new.
	min := 1000000.
	max := 999000000.
	delta := max - min.
	self numberOfCores timesRepeat:[
		testData add: min + delta atRandom].
	^testData! !

!RVMMulticoreSample methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:20'!
totalYields
^ self cpuCoreStats yieldCounts sum! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:08'!
unforcedInterruptCheckCounts
	"RVMMulticoreSample capture yieldCounts"
	"Answer the collection of yields during this sample"
	
	^self cpuCoreStats unforcedInterruptCheckCounts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:17'!
yieldCounts
	"RVMMulticoreSample capture yieldCounts"
	"Answer the collection of yields during this sample"
	
	^self cpuCoreStats yieldCounts! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:18'!
yieldCountsAverage
	"RVMMulticoreSample capture yieldCountsAverage"
	"Answer the average of yields during this sample"
	
	^self cpuCoreStats yieldCounts average asFloat! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'dmu 1/1/1970 00:16'!
yieldFrequencies
	"RVMMulticoreSample capture yieldFrequencies"
	
	^self cpuCoreStats yieldFrequencies! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 16:36'!
yieldsPerSecond
	"RVMMulticoreSample capture yieldsPerSecond"
	"Answer the collection of ratios of yields  per second during this sample"
	
	^self cpuCoreStats yieldsPerSecond! !

!RVMMulticoreSample methodsFor: 'measuring' stamp: 'ssa 8/14/2008 17:15'!
yieldsPerSecondAverage
	"RVMMulticoreSample capture yieldsPerSecondAverage"
	"Answer the average of ratios of yields  per second during this sample"
	
	^self cpuCoreStats yieldsPerSecond average asFloat! !

RVMObject subclass: #RVMObjectCoordinate
	instanceVariableNames: 'object mutability activity coreIndex coreCoordinate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Misc'!

!RVMObjectCoordinate class methodsFor: 'instance creation' stamp: 'ssa 11/16/2008 01:27'!
for: anObject
	"Answer the coordinate for this object"
	| coord |
	coord := self new.
	coord object: anObject.
	^coord! !

!RVMObjectCoordinate methodsFor: 'accessing' stamp: 'ads 1/30/2011 15:23'!
activity
	"Answer the value of activity, whether or not you want the baton passed on a message send"

	^RVMOperations getPassTheBatonOf: self object! !

!RVMObjectCoordinate methodsFor: 'accessing' stamp: 'ssa 11/16/2008 02:11'!
coreCoordinate
	"Answer the value of coreCoordinate"

	^(self coreIndex + 1 asPointWithinRectangleWithExtent: RVMMonitor extent) ! !

!RVMObjectCoordinate methodsFor: 'accessing' stamp: 'ads 1/30/2011 15:23'!
coreIndex
	"Answer the value of coreIndex"

	^(RVMOperations getCoreHolding: self object)! !

!RVMObjectCoordinate methodsFor: 'testing' stamp: 'ssa 11/13/2008 10:26'!
isActive

	^self activity = true! !

!RVMObjectCoordinate methodsFor: 'testing' stamp: 'ssa 11/13/2008 10:26'!
isPassive

	^self activity = false! !

!RVMObjectCoordinate methodsFor: 'testing' stamp: 'ssa 11/13/2008 10:26'!
isReadMostly

	^self mutability = false! !

!RVMObjectCoordinate methodsFor: 'testing' stamp: 'ssa 11/13/2008 10:25'!
isReadWrite

	^self mutability = true! !

!RVMObjectCoordinate methodsFor: 'moving' stamp: 'ads 1/30/2011 15:23'!
makeActive
RVMOperations isRVM ifTrue:[
	RVMOperations setCoordinatesOf: self object toCore: self coreIndex  readWrite:  self mutability].
	self changed! !

!RVMObjectCoordinate methodsFor: 'moving' stamp: 'ads 1/30/2011 15:23'!
makePassive

RVMOperations isRVM ifTrue:[
	RVMOperations setCoordinatesOf: self object toCore: self coreIndex readWrite: self mutability ].
	self changed! !

!RVMObjectCoordinate methodsFor: 'moving' stamp: 'ads 1/30/2011 15:23'!
makeReadMostly
RVMOperations isRVM ifTrue:[
	RVMOperations setCoordinatesOf: self object toCore: self coreIndex readWrite: false ].
	self changed! !

!RVMObjectCoordinate methodsFor: 'moving' stamp: 'ads 1/30/2011 15:23'!
makeReadWrite
RVMOperations isRVM ifTrue:[
	RVMOperations setCoordinatesOf: self object toCore: self coreIndex readWrite: true].
	self changed! !

!RVMObjectCoordinate methodsFor: 'moving' stamp: 'ssa 11/16/2008 02:23'!
moveToCoordinate: aPoint

	self moveToCore:  (aPoint y -1 * RVMMonitor extent x + aPoint x) -1 .
	self changed:#coreCoordinate! !

!RVMObjectCoordinate methodsFor: 'moving' stamp: 'ads 1/30/2011 15:23'!
moveToCore: anIndex

RVMOperations isRVM ifTrue:[
	RVMOperations setCoordinatesOf: self object toCore: (anIndex max:0) readWrite: self mutability ].
	self changed! !

!RVMObjectCoordinate methodsFor: 'accessing' stamp: 'ads 1/30/2011 15:23'!
mutability
	"Answer the value of mutability, whether you are readWrite (true) or readMostly (false)"

	^RVMOperations getReadWriteOf: self object! !

!RVMObjectCoordinate methodsFor: 'accessing' stamp: 'ssa 11/13/2008 10:29'!
object
	"Answer the value of object"

	object isNil ifTrue:[self object: nil].
	^ object! !

!RVMObjectCoordinate methodsFor: 'accessing' stamp: 'ssa 11/13/2008 10:29'!
object: anObject
	"Set the value of object"

	object := anObject! !

RVMObject subclass: #RVMObjectTableStats
	instanceVariableNames: 'entryCount allocatedEntryCount entriesFreedSinceLastQuery allocationsSinceLastQuery'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:20'!
allocatedEntryCount
	"Answer the number of allocated entries in this object table"
	allocatedEntryCount isNil ifTrue:[^0].
	^allocatedEntryCount! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:20'!
allocatedEntryCount: count
	"Set the number of allocated entries in this object table"
	allocatedEntryCount := count! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'dmu 8/15/2008 21:43'!
allocationsSinceLastQuery
	"Answer the allocations made since the last query"
	allocationsSinceLastQuery isNil ifTrue:[^0].
	^allocationsSinceLastQuery! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'dmu 8/15/2008 21:44'!
allocationsSinceLastQuery: count
	"Set the allocations made since the last query"
	allocationsSinceLastQuery := count! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'dmu 8/15/2008 21:45'!
entriesFreedSinceLastQuery
	"Answer the allocations freed since the last query"
	entriesFreedSinceLastQuery isNil ifTrue:[^0].
	^entriesFreedSinceLastQuery! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'dmu 8/15/2008 21:46'!
entriesFreedSinceLastQuery: count
	"Set the allocations freed since the last query"
	entriesFreedSinceLastQuery := count! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:19'!
entryCount
	"Answer the number of entries in this object table"
	entryCount isNil ifTrue:[^0].
	^entryCount! !

!RVMObjectTableStats methodsFor: 'acessing' stamp: 'ssa 7/21/2008 10:19'!
entryCount: count
	"Set the number of entries in this object table"
	entryCount := count! !

RVMObject subclass: #RVMSampleFlags
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:27'!
allCoreStats ^ 1 bitShift: 4! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:27'!
allCores ^ 1 bitShift: 0! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:28'!
bufferedMessageStats ^ 1 bitShift: 17! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:28'!
bytecodes ^ 1 bitShift: 21! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:27'!
coreCoords ^ 1 bitShift: 14! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:28'!
cpuCoreStats ^ 1 bitShift: 3! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:29'!
cycleCounts ^ 1 bitShift: 23! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:29'!
cycles ^ 1 bitShift: 8! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:29'!
dontPassTheBatonDefault ^ 1 bitShift: 5! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:30'!
everything ^ SmallInteger maxVal! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:31'!
fence  ^  1 bitShift: 6! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:31'!
gcStats ^ 1 bitShift: 19! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:32'!
heapStats ^ 1 bitShift: 20! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:32'!
interactionStats ^ 1 bitShift: 13! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:32'!
interpreterLoopStats ^ 1 bitShift: 27! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:32'!
interpreterStats ^ 1 bitShift: 11! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:32'!
interruptChecks ^ 1 bitShift: 24! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:33'!
memorySystemStats  ^ 1 bitShift: 10! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:33'!
messageNames ^ 1 bitShift: 2! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:33'!
messageStats  ^ 1 bitShift: 9! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:33'!
millisecs  ^ 1 bitShift: 7! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:33'!
movedMutatedObjectStats ^ 1 bitShift: 25! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:34'!
mutexStats ^ 1 bitShift: 26
! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:34'!
objectTableStats  ^ 1 bitShift: 12! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:34'!
receiveCycles ^ 1 bitShift: 18! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:34'!
receiveTallies ^ 1 bitShift: 16! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:34'!
runMask ^ 1 bitShift: 1! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:35'!
sendTallies ^ 1 bitShift: 15! !

!RVMSampleFlags class methodsFor: 'flag values' stamp: 'dmu 4/10/2009 17:35'!
yieldCount ^  1 bitShift: 22! !

RVMObject subclass: #RVMSqueakInterpreterStats
	instanceVariableNames: 'bytecodesExecuted yieldCount millisecondsRunning millisecondsWaiting cyclesRunning cyclesWaiting contextChangeCount interruptCheckCount unforcedInterruptCheckCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Monitoring'!

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:27'!
bytecodesExecuted
	"Answer the number of bytecodes processed since last query"
	bytecodesExecuted isNil ifTrue:[^0].
	^bytecodesExecuted! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:27'!
bytecodesExecuted: count
	"Set the number of bytecodes processed since last query"
	bytecodesExecuted := count! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:02'!
contextChangeCount
	"Answer the value of contextChangeCount"

	contextChangeCount isNil ifTrue:[self contextChangeCount: 0].
	^contextChangeCount! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:10'!
contextChangeCount: anObject
	"Set the value of contextChangeCount"

	contextChangeCount := anObject! !

!RVMSqueakInterpreterStats methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:21'!
cyclesRunWaitRatio
	"Answer the ratio between my cyclesRunning and cyclesWaiting"
	
	^self cyclesWaiting = 0 ifTrue:[0] ifFalse:[self cyclesRunning / (1 max: self cyclesWaiting) asFloat]! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:02'!
cyclesRunning
	"Answer the value of cyclesRunning"

	cyclesRunning isNil ifTrue:[self cyclesRunning: 0].
	^cyclesRunning! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:13'!
cyclesRunning: anObject
	"Set the value of cyclesRunning"

	cyclesRunning := anObject! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:02'!
cyclesWaiting
	"Answer the value of cyclesWaiting"

	cyclesWaiting isNil ifTrue:[self cyclesWaiting: 0].
	^cyclesWaiting! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:13'!
cyclesWaiting: anObject
	"Set the value of cyclesWaiting"

	cyclesWaiting := anObject! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:03'!
interruptCheckCount
^ interruptCheckCount! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:03'!
interruptCheckCount: x
 interruptCheckCount := x! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:03'!
millisecondsRunning
	"Answer the value of millisecondsRunning"

	millisecondsRunning isNil ifTrue:[self millisecondsRunning: 0].
	^millisecondsRunning! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:24'!
millisecondsRunning: anObject
	"Set the value of msRunning"

	millisecondsRunning := anObject! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:01'!
millisecondsWaiting
	"Answer the value of msWaiting"

	millisecondsWaiting isNil ifTrue:[self millisecondsWaiting: 0].
	^millisecondsWaiting! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 8/15/2008 21:25'!
millisecondsWaiting: anObject
	"Set the value of msWaiting"

	millisecondsWaiting := anObject! !

!RVMSqueakInterpreterStats methodsFor: 'measuring' stamp: 'dmu 9/17/2010 15:21'!
msRunWaitRatio
	"Answer the ratio between my millisecondsRunning and millisecondsWaiting"
	
	^self millisecondsWaiting = 0 ifTrue:[0] ifFalse:[self millisecondsRunning / (1 max: self millisecondsWaiting) asFloat]! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:04'!
unforcedInterruptCheckCount
 ^ unforcedInterruptCheckCount! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 1/1/1970 00:03'!
unforcedInterruptCheckCount: x
 unforcedInterruptCheckCount := x! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 1/1/1970 00:03'!
yieldCount
	"Answer the value of yieldCount"

	yieldCount isNil ifTrue:[self yieldCount: 0].
	^yieldCount! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'ssa 8/13/2008 18:13'!
yieldCount: anObject
	"Set the value of yieldCount"

	yieldCount := anObject! !

!RVMSqueakInterpreterStats methodsFor: 'accessing' stamp: 'dmu 9/17/2010 15:21'!
yieldFrequency
	"Answer the value of yieldCount/contextChangeCount"
	^self  contextChangeCount = 0 ifTrue: [0] ifFalse: [(self yieldCount / (1 max: self contextChangeCount)) asFloat]! !

Object subclass: #RVMOperations
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM'!

!RVMOperations class methodsFor: 'enumerating a heap' stamp: 'dmu 6/8/2010 23:10'!
allObjectsInHeap: rank isReadWrite: trueForReadWriteFalseForReadMostly
	"return array of all objects in either read/write or read-mostly heap specified by rank and trueForReadWriteFalseForReadMostly"
	"RVMOperations allObjectsInHeap: 0 isReadWrite: true"
	<primitive: 'primitiveAllObjectsInHeap' module: 'RVMPlugin'>
	^ nil! !

!RVMOperations class methodsFor: 'debugging VM' stamp: 'StefanMarr 3/19/2012 16:08'!
breakpoint
	<primitive: 'primitiveBreakpoint' module: 'RVMPlugin'>
	self primitiveFailed! !

!RVMOperations class methodsFor: 'cores' stamp: 'dmu 6/8/2010 23:11'!
coreCount
  "RVMOperations coreCount"
  <primitive: 'primitiveCoreCount' module: 'RVMPlugin'>
  ^ 1! !

!RVMOperations class methodsFor: 'situating an object' stamp: 'dmu 6/8/2010 23:30'!
for: anObject setRankTo: rank
	<primitive: 'primitiveSetCoordinatesFor' module: 'RVMPlugin'>
	self primitiveFailed! !

!RVMOperations class methodsFor: 'situating an object' stamp: 'dmu 6/8/2010 23:31'!
for: anObject setRankTo: rank isReadWrite: isRW
	<primitive: 'primitiveSetCoordinatesFor' module: 'RVMPlugin'>
	self primitiveFailed! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 1/24/2011 12:47'!
getCoreHolding: anObject
	^(self primitivGetCoreHolding: anObject)! !

!RVMOperations class methodsFor: 'processes' stamp: 'dmu 6/8/2010 23:18'!
getCoreIAmRunningOn
	"RVMOperations getCoreIAmRunningOn"
	<primitive: 'primitiveGetCoreIAmRunningOn' module: 'RVMPlugin' >
  	^ -1! !

!RVMOperations class methodsFor: 'situating an object' stamp: 'dmu 6/8/2010 23:17'!
getCoreOf: anObject
	"RVMOperations getCoreOf: 2@3"
	<primitive: 'primitiveGetCore' module: 'RVMPlugin'>
	^ -1! !

!RVMOperations class methodsFor: 'tracing' stamp: 'dmu 6/8/2010 23:25'!
getCoreTrace
	"RVMOperations getCoreTrace"
	<primitive: 'primitiveTraceCores' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 1/24/2011 12:57'!
getEmergencySemaphore
	"When you hit shift-control-a, the VM will signal the emergency semaphore, if it has been set to a semaphore"
	"self getEmergencySemaphore signal"
	<primitive: 'primitiveEmergencySemaphore' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'using extra word' stamp: 'dmu 6/14/2010 15:27'!
getExtraPreheaderWordOf: anObject
	"Returns the extra preheader word of the argument."
	"RVMOperations getExtraPreheaderWordOf: 3@4"
	<primitive: 'primitiveGetExtraPreheaderWord' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'VM statistics' stamp: 'sm 10/27/2010 22:09'!
getMainRank
	"Get a sample from the VM and read out the main rank"
	| sample |
	sample := self sampleRVM.
	^ ((sample at: 6) at: 2) at: 6! !

!RVMOperations class methodsFor: 'situating an object' stamp: 'dmu 6/8/2010 23:47'!
getMutabilityOf: anObject
	"Return true if argument is in a read/write heap, false if in read-mostly heap"
	<primitive: 'primitiveGetMutability' module: 'RVMPlugin'>
	^ true! !

!RVMOperations class methodsFor: 'tracing' stamp: 'dmu 6/8/2010 23:26'!
getMutatedReplicatedObjectsTrace
	<primitive: 'primitiveTraceMutatedReplicatedObjects' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 1/24/2011 12:55'!
getReadWriteOf: anObject
<primitive: 'primitiveGetMutability' module: 'RVMPlugin'>
^ true! !

!RVMOperations class methodsFor: 'processes' stamp: 'dmu 6/8/2010 23:44'!
getRunMask
	"Return an integer with ones corresponding to cores RVM is allowed to use: 1 means core 0, 3 means cores 0 and 1, etc."
	"RVMOperations getRunMask printStringBase: 16"
	<primitive: 'primitiveRunMask' module: 'RVMPlugin'>
	^ -1! !

!RVMOperations class methodsFor: 'processes' stamp: 'dmu 6/9/2010 00:07'!
getRunningProcessesByCore
	"Returns array of running processes, indexed by core (+1)"
	<primitive: 'primitiveRunningProcessByCore' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/6/2011 10:09'!
initialize
	(Process hasInstVarNamed: 'hostCore') ifFalse: [
		Process addInstVarNamed: 'hostCore'.].
	(Process hasInstVarNamed: 'coreMask') ifFalse: [
		Process addInstVarNamed: 'coreMask'.].! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 2/2/2011 14:54'!
isRVM
	"RVMOperations isRVM"
	"Am I actually running on the Tilera64?"
	^(self getCoreHolding: nil) ~= -1! !

!RVMOperations class methodsFor: 'rearranging objects' stamp: 'dmu 6/8/2010 23:58'!
moveAllToReadMostlyHeaps
	"Move all objects into read-mostly heaps"
	<primitive: 'primitiveMoveAllToReadMostlyHeaps' module: 'RVMPlugin'>
	self primitiveFailed! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 1/24/2011 12:47'!
primitivGetCoreHolding: anObject
	<primitive: 'primitiveGetCore' module: 'RVMPlugin'>
	^ -1! !

!RVMOperations class methodsFor: 'debugging VM' stamp: 'dmu 6/9/2010 00:02'!
printExecutionTrace
	<primitive: 'primitivePrintExecutionTrace' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'debugging VM' stamp: 'dmu 6/9/2010 00:01'!
printOnConsole: aString
	"Prints a string onto the console"
	<primitive: 'primitivePrint' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 2/2/2011 14:54'!
printPrefixFor: anObject
	"RVMOperations printPrefixFor: Array "
	"Answer a string to use as a prefix in Inspectors and Debuggers when printing an object"
	| index readWrite |
	(anObject isKindOf: SmallInteger) ifTrue:[^''].
	(anObject isKindOf: Boolean) ifTrue:[^''].
	(anObject isNil) ifTrue:[^''].
	index := (self getCoreHolding: anObject) printString.
	readWrite := (self getReadWriteOf: anObject) ifTrue: ['rw'] ifFalse: ['rm'].
	
	
	^'<',readWrite,index,'> '! !

!RVMOperations class methodsFor: 'debugging VM' stamp: 'dmu 6/9/2010 00:00'!
printStack
	"Prints Smalltalk stack to console"
	<primitive: 'primitivePrintStack' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'debugging VM' stamp: 'dmu 6/9/2010 00:01'!
printVMStatistics
	<primitive: 'primitivePrintStats' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'VM statistics' stamp: 'dmu 6/8/2010 23:04'!
sampleRVM
	"Grab some data from the VM and return it"
	<primitive: 'primitiveSampleRVM' module: 'RVMPlugin'>
	"RVMOperations sampleRVM"
	^ nil! !

!RVMOperations class methodsFor: 'VM statistics' stamp: 'dmu 6/8/2010 23:40'!
sampleSpecificData: flags
	"Grab some data from the VM and return it"
	"Bit offsets for flags; bit 0 is 1, bit 1 is 2, bit 2 is 4, etc.
	
	allCores, // 0
	runMask,
	messageNames,
	cpuCoreStats,
	allCoreStats,
	fence,

	millisecs, // 6
	cycles,
	messageStats,
	memorySystemStats,
    	interpreterStats,
    	objectTableStats,
    	interactionStats,

   	 coreCoords, // 14
    	sendTallies,
    	receiveTallies,
    	bufferedMessageStats,
    	receiveCycles,

    	// memory system
    	gcStats, // 19
    	heapStats,

    	// interpreter
    	bytecodes, // 20
    	yieldCount,
    	cycleCounts,
    	interruptChecks,
    	movedMutatedObjectStats,
    	mutexStats,
   	 interpreterLoopStats // 26"

	<primitive: 'primitiveSampleRVM' module: 'RVMPlugin'>
	^ nil! !

!RVMOperations class methodsFor: 'as yet unclassified' stamp: 'ads 1/24/2011 12:57'!
setEmergencySemaphore: aSemaOrNil
	"When you hit shift-control-a, the VM will signal the emergency semaphore, if it has been set to a semaphore"
	<primitive: 'primitiveEmergencySemaphore' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'using extra word' stamp: 'dmu 6/14/2010 15:27'!
setExtraPreheaderWordOf: anObject to: newContents
	"Sets the extra preheader word of the first argument."
	"|n| n := 3@4.
	RVMOperations setExtraPreheaderWordOf: n to: 17.
	RVMOperations getExtraPreheaderWordOf: n"
	<primitive: 'primitiveSetExtraPreheaderWord' module: 'RVMPlugin'>! !

!RVMOperations class methodsFor: 'using extra word' stamp: 'dmu 6/9/2010 00:05'!
setExtraWordSelector: aSelector
	"returns the old one"
	<primitive: 'primitiveSetExtraWordSelector' module: 'RVMPlugin'>
	^self primitiveFailed! !

!RVMOperations class methodsFor: 'processes' stamp: 'dmu 6/8/2010 23:46'!
setRunMask: anInteger
	"Sets run mask to argument and returns old run mask.
	Only cores set in run mask will be used to run Smalltalk processes."
	"RVMOperations setRunMask: 1"
	<primitive: 'primitiveRunMask' module: 'RVMPlugin'>
	^ -1! !

!RVMOperations class methodsFor: 'rearranging objects' stamp: 'dmu 6/8/2010 23:55'!
shuffleToHeapsFrom: firstHeap to: lastHeap
	"Deal out all objects in the system into heaps from firstHeap through lastHeap in round-robin fashion so that adjacent objects wind up in different heaps"
	<primitive: 'primitiveShuffle' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'rearranging objects' stamp: 'dmu 6/8/2010 23:56'!
shuffleToHeapsFrom: firstHeap to: lastHeap movingReadWriteObjectsToReadMostlyHeap: rwToRM  movingReadMostlyObjectsToReadWriteHeap: rmToRw
	"Deal out all objects in the system into heaps from firstHeap through lastHeap in round-robin fashion so that adjacent objects wind up in different heaps"
	"If rwToRM, move objects in read/write heaps to read-mostly heaps."
	"If rmToRW, move objects in read-mostly heaps to read/write heaps."
	<primitive: 'primitiveShuffle' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'rearranging objects' stamp: 'dmu 6/8/2010 23:57'!
spreadToHeapsFrom: firstHeap to: lastHeap
	"Spread out all objects in the system into heaps from firstHeap through lastHeap so that each heap ends up about equally full."
	<primitive: 'primitiveSpread' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'rearranging objects' stamp: 'dmu 6/8/2010 23:57'!
spreadToHeapsFrom: firstHeap to: lastHeap movingReadWriteObjectsToReadMostlyHeap: rwToRM  movingReadMostlyObjectsToReadWriteHeap: rmToRw
	"Spread out all objects in the system into heaps from firstHeap through lastHeap so that each heap ends up about equally full."
	"If rwToRM, move objects in read/write heaps to read-mostly heaps."
	"If rmToRW, move objects in read-mostly heaps to read/write heaps."
	<primitive: 'primitiveSpread' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'tracing' stamp: 'dmu 6/8/2010 23:22'!
startCoreTrace: size
	"RVMOperations startCoreTrace: 1000"
	<primitive: 'primitiveTraceCores' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'tracing' stamp: 'dmu 6/8/2010 23:27'!
startMutatedReplicatedObjectsTrace: size
	<primitive: 'primitiveTraceMutatedReplicatedObjects' module: 'RVMPlugin'>
	^ self primitiveFailed! !

!RVMOperations class methodsFor: 'processes' stamp: 'dmu 6/9/2010 00:04'!
thisProcess
	"Return the process that is running; RVM does not use ActiveProcess for this state, since many may be running"
	<primitive: 'primitiveThisProcess'  module: 'RVMPlugin'>
	^ Processor activeProcess! !

Object subclass: #RVMReadersWriterLock
	instanceVariableNames: 'readerCount readerCountInterlock readVsWriteInterlock writingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RoarVM-Misc'!

!RVMReadersWriterLock commentStamp: 'dmu 11/12/2010 17:26' prior: 0!
I provide synchronization for multiple readers or one writer.
I am not very clever.

readerCount: how many readers are currently reading
readerCountInterlock: protects readerCount
readVsWriteInterlock: ensures that reading & writing do not happen at the same time, also ensures only one writer at a time

Usage:

x := RVMReadersWriterLock new.

x guardForReading: [ "read the data" ].
x guardForWriting: [ "write the data" ].!

!RVMReadersWriterLock class methodsFor: 'as yet unclassified' stamp: 'dmu 11/12/2010 18:08'!
crudeTest
	| x g p |
	"RVMReadersWriterLock  crudeTest"
	"Flashes dots in upper row when reading, flashes dots in lower row when writing."
	x := RVMReadersWriterLock new.
	g := 0@0.
	1 to: 10 do: [:i| 
		[	[g x = 0] whileTrue.
			p :=  (100@100) + ( (5*i) @ 0).
			100 timesRepeat: [
				x guardForReading: [
					(Delay forMilliseconds: 10) wait.
					Display drawDot5At: p.
					(Delay forMilliseconds: 10) wait.
					Display eraseDot5At: p.
				]
			]
		] copy fixTemps fork.
	].
	1 to: 4 do: [:i| 
		[     [g x = 0] whileTrue.
			p :=  (100@110) + ( (5*i) @ 0).
			100 timesRepeat: [
				x guardForWriting: [
					(Delay forMilliseconds: 10) wait.
					Display drawDot5At: p.
					(Delay forMilliseconds: 10) wait.
					Display eraseDot5At: p.
				]
			]
		] copy fixTemps fork.
	].
	g x: 1! !

!RVMReadersWriterLock methodsFor: 'private' stamp: 'dmu 11/13/2010 17:41'!
finishedReading
	self isRunningWritingProcess ifTrue: [^ self].
	readerCountInterlock critical: [
		readerCount > 1  ifTrue: [readerCount := readerCount - 1] ifFalse: [
	        readerCount = 1 ifFalse: [self error: 'oops'] ifTrue: [
			readerCount := 0.
			 readVsWriteInterlock signal
		]].
	]! !

!RVMReadersWriterLock methodsFor: 'private' stamp: 'dmu 11/13/2010 16:29'!
finishedWriting
	writingProcess := nil.
	readVsWriteInterlock signal! !

!RVMReadersWriterLock methodsFor: 'public' stamp: 'dmu 11/12/2010 17:31'!
guardForReading: blockWhichReads
	self startingToRead.
	^ blockWhichReads ensure: [self finishedReading]! !

!RVMReadersWriterLock methodsFor: 'public' stamp: 'dmu 11/12/2010 17:31'!
guardForWriting: blockWhichWrites
	self startingToWrite.
	^ blockWhichWrites ensure: [self finishedWriting]! !

!RVMReadersWriterLock methodsFor: 'private' stamp: 'dmu 11/13/2010 16:28'!
initialize
	readerCount := 0.
	readerCountInterlock := Semaphore forMutualExclusion.
	readVsWriteInterlock := Semaphore forMutualExclusion.
	writingProcess := nil! !

!RVMReadersWriterLock methodsFor: 'private' stamp: 'dmu 11/13/2010 16:33'!
isRunningWritingProcess
	^ Processor thisProcess == writingProcess! !

!RVMReadersWriterLock methodsFor: 'private' stamp: 'dmu 11/13/2010 17:42'!
startingToRead
	self isRunningWritingProcess  ifTrue: [^ self].
	readerCountInterlock critical: [
		readerCount > 0  ifTrue: [readerCount := readerCount + 1] ifFalse: [
	        readerCount = 0 ifFalse: [self error: 'oops'] ifTrue: [
			readVsWriteInterlock wait.
			readerCount := 1.
		]].
	]! !

!RVMReadersWriterLock methodsFor: 'private' stamp: 'dmu 6/13/2011 23:53'!
startingToWrite
	writingProcess == Processor thisProcess ifTrue: [self error: 'recursive write'].
	readVsWriteInterlock wait.
	writingProcess := Processor thisProcess! !
RVMOperations initialize!
